#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Akini Ross (@akinross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_aaa_security_default_settings
short_description: Manage AAA Key Rings (pki:Ep)
description:
- Manage AAA Key Rings on Cisco ACI fabrics.
options:
  password_strength_check:
    description:
    - Enable password strength check.
    - Use C(true) to enable and C(false) to disable.
    - The APIC defaults to C(true) when unset during creation.
    type: bool
  password_strength_profile:
    description:
    - The password strength profile (aaa:PwdStrengthProfile).
    type: dict
    suboptions:
      enable:
        description:
        - Enable or disable password strength profile.
        - Use C(true) to enable and C(false) to disable.
        type: bool
        required: true
      type:
        description:
        - The type of the password strength profile.
        - The APIC defaults to C(any_three) when unset during creation.
        type: str
        choices: [ custom, any_three ]
      min_length:
        description:
        - The minimum length of the password.
        - The APIC defaults to C(8) when unset during creation.
        type: int
        aliases: [ minimum_length, min ]
      max_length:
        description:
        - The maximum length of the password.
        - The APIC defaults to C(64) when unset during creation.
        type: int
        aliases: [ maximum_length, max ]
      class_flags:
        description:
        - The class flags of the password strength profile.
        - At least 3 class flags must be specified.
        - This attribute is only applicable when type is set to O(password_strength_profile.type=custom).
        - The APIC defaults to C(digits,lowercase,uppercase) when unset during creation.
        type: list
        elements: str
        choices: [ digits, lowercase, specialchars, uppercase ]
        aliases: [ flags ]
  password_change:
    description:
    - The password change interval (aaa:PwdProfile).
    type: dict
    suboptions:
      enable:
        description:
        - Enforce password change interval.
        - Use C(true) to enable and C(false) to disable.
        - The APIC defaults to C(true) when unset during creation.
        type: bool
      interval:
        description:
        - The password change interval in hours.
        - The APIC defaults to C(48) when unset during creation.
        type: int
      allowed_changes:
        description:
        - The number of changes allowed within the change interval.
        - The APIC defaults to C(2) when unset during creation.
        type: int
      minimum_period:
        description:
        - The minimum period between password changes in hours.
        - The APIC defaults to C(24) when unset during creation.
        type: int
        aliases: [ minimum_period_between_password_changes, min_period ]
      history_storage_amount:
        description:
        - The number of recent user passwords to store.
        - The APIC defaults to C(5) when unset during creation.
        type: int
        aliases: [ history, amount ]
  lockout:
    description:
    - Lockout behaviour after multiple failed login attempts (aaa:BlockLoginProfile).
    type: dict
    suboptions:
      enable:
        description:
        - Enable lockout behaviour.
        - Use C(true) to enable and C(false) to disable.
        - The APIC defaults to C(false) when unset during creation.
        type: bool
      max_attempts:
        description:
        - The maximum number of failed attempts before user is locked out.
        - The APIC defaults to C(5) when unset during creation.
        type: int
        aliases: [ max_failed_attempts, failed_attempts, attempts ]
      window:
        description:
        - The time period in which consecutive attempts were failed in minutes.
        - The APIC defaults to C(5) when unset during creation.
        type: int
        aliases: [ max_failed_attempts_window, failed_attempts_window ]
      duration:
        description:
        - The duration of lockout in minutes.
        - The APIC defaults to C(60) when unset during creation.
        type: int
  web_token:
    description:
    - The web token related configuration (pki:WebTokenData).
    type: dict
    suboptions:
      timeout:
        description:
        - The web token timeout in seconds.
        - The APIC defaults to C(600) when unset during creation.
        type: int
      idle_timeout:
        description:
        - The web/console (SSH/Telnet) session idle timeout in seconds.
        - The APIC defaults to C(1200) when unset during creation.
        type: int
        aliases: [ session_idle_timeout ]
      validity_period:
        description:
        - The maximum validity period in hours.
        - The APIC defaults to C(24) when unset during creation.
        type: int
        aliases: [ maximum_validity_period ]
      refresh:
        description:
        - Include refresh in session records.
        - Use C(true) to include and C(false) to exclude.
        - The APIC defaults to C(false) when unset during creation.
        type: bool
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(pki:Ep).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Set AAA Security Default Settings
  cisco.aci.aci_aaa_security_default_settings:
    host: apic
    username: admin
    password: SomeSecretPassword
    password_strength_check: true
    password_strength_profile:
      enable: true
      type: custom
      min_length: 10
      max_length: 60
      class_flags:
        - digits
        - lowercase
        - specialchars
        - uppercase
    password_change:
      enable: true
      interval: 49
      allowed_changes: 6
      minimum_period_between_password_changes: 25
      history_storage_amount: 6
    lockout:
      enable: true
      max_attempts: 6
      window: 6
      duration: 61
    web_token:
      timeout: 601
      idle_timeout: 1201
      validity_period: 23
      refresh: true
    state: present
  delegate_to: localhost

- name: Set AAA Security Default Settings to Default Values
  cisco.aci.aci_aaa_security_default_settings:
    host: apic
    username: admin
    password: SomeSecretPassword
    password_strength_check: true
    password_strength_profile:
      enable: false
    password_change:
      enable: true
      interval: 48
      allowed_changes: 2
      minimum_period_between_password_changes: 24
      history_storage_amount: 5
    lockout:
      enable: false
      max_attempts: 5
      window: 5
      duration: 60
    web_token:
      timeout: 600
      idle_timeout: 1200
      validity_period: 24
      refresh: false
    state: present
  delegate_to: localhost

- name: Query AAA Security Default Settings
  cisco.aci.aci_aaa_security_default_settings:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        password_strength_check=dict(type="bool", no_log=False),
        password_strength_profile=dict(
            type="dict",
            no_log=False,
            options=dict(
                enable=dict(type="bool", required=True),
                type=dict(type="str", choices=["custom", "any_three"]),
                min_length=dict(type="int", aliases=["minimum_length", "min"]),
                max_length=dict(type="int", aliases=["maximum_length", "max"]),
                class_flags=dict(type="list", elements="str", choices=["digits", "lowercase", "specialchars", "uppercase"], aliases=["flags"]),
            ),
        ),
        password_change=dict(
            type="dict",
            no_log=False,
            options=dict(
                enable=dict(type="bool"),
                interval=dict(type="int"),
                allowed_changes=dict(type="int"),
                minimum_period=dict(type="int", aliases=["minimum_period_between_password_changes", "min_period"]),
                history_storage_amount=dict(type="int", aliases=["history", "amount"]),
            ),
        ),
        lockout=dict(
            type="dict",
            options=dict(
                enable=dict(type="bool"),
                max_attempts=dict(type="int", aliases=["max_failed_attempts", "failed_attempts", "attempts"]),
                window=dict(type="int", aliases=["max_failed_attempts_window", "failed_attempts_window"]),
                duration=dict(type="int"),
            ),
        ),
        web_token=dict(
            type="dict",
            no_log=False,
            options=dict(
                timeout=dict(type="int"),
                idle_timeout=dict(type="int", aliases=["session_idle_timeout"]),
                validity_period=dict(type="int", aliases=["maximum_validity_period"]),
                refresh=dict(type="bool"),
            ),
        ),
        state=dict(type="str", default="present", choices=["present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )
    aci = ACIModule(module)

    password_strength_check = aci.boolean(module.params.get("password_strength_check"))
    password_strength_profile = module.params.get("password_strength_profile")
    password_change = module.params.get("password_change")
    lockout = module.params.get("lockout")
    web_token = module.params.get("web_token")
    state = module.params.get("state")
    name_alias = module.params.get("name_alias")

    aci_class = "aaaUserEp"
    child_classes = ["aaaPwdStrengthProfile", "aaaPwdProfile", "aaaBlockLoginProfile", "pkiWebTokenData"]

    aci.construct_url(
        root_class=dict(
            aci_class=aci_class,
            aci_rn="userext",
        ),
        child_classes=child_classes,
    )
    aci.get_existing()

    if state == "present":
        child_configs = []

        class_config = dict(
            pwdStrengthCheck=password_strength_check,
            nameAlias=name_alias,
        )

        if password_strength_profile:
            if password_strength_profile.get("enable"):
                child_configs.append(
                    dict(
                        aaaPwdStrengthProfile=dict(
                            attributes=dict(
                                pwdStrengthTestType=password_strength_profile.get("type"),
                                pwdMinLength=password_strength_profile.get("min_length"),
                                pwdMaxLength=password_strength_profile.get("max_length"),
                                pwdClassFlags=",".join(sorted(password_strength_profile.get("class_flags"))),
                            ),
                        ),
                    ),
                )
            # Delete existing aaaPwdStrengthProfile if enable is set to false and it exists
            # This is done for setting the correct output for changed state
            elif len(aci.existing) > 0 and len(aci.existing[0].get("aaaUserEp", {}).get("children", {})) > 3:
                for child in aci.existing[0].get("aaaUserEp", {}).get("children", {}):
                    if "aaaPwdStrengthProfile" in child.keys():
                        child_configs.append(dict(aaaPwdStrengthProfile=dict(attributes=dict(status="deleted"))))
                        break

        if password_change:
            child_configs.append(
                dict(
                    aaaPwdProfile=dict(
                        attributes=dict(
                            changeDuringInterval=aci.boolean(password_change.get("enable"), "enable", "disable"),
                            changeInterval=password_change.get("interval"),
                            changeCount=password_change.get("allowed_changes"),
                            noChangeInterval=password_change.get("minimum_period"),
                            historyCount=password_change.get("history_storage_amount"),
                        ),
                    ),
                ),
            )

        if lockout:
            child_configs.append(
                dict(
                    aaaBlockLoginProfile=dict(
                        attributes=dict(
                            enableLoginBlock=aci.boolean(lockout.get("enable"), "enable", "disable"),
                            maxFailedAttempts=lockout.get("max_attempts"),
                            maxFailedAttemptsWindow=lockout.get("window"),
                            blockDuration=lockout.get("duration"),
                        ),
                    ),
                ),
            )

        if web_token:
            child_configs.append(
                dict(
                    pkiEp=dict(
                        attributes=dict(descr=""),
                        children=[
                            dict(
                                pkiWebTokenData=dict(
                                    attributes=dict(
                                        webtokenTimeoutSeconds=web_token.get("timeout"),
                                        uiIdleTimeoutSeconds=web_token.get("idle_timeout"),
                                        maximumValidityPeriod=web_token.get("validity_period"),
                                        sessionRecordFlags="login,logout,refresh" if web_token.get("refresh") else "login,logout",
                                    ),
                                ),
                            ),
                        ],
                    ),
                ),
            )

        aci.payload(
            aci_class=aci_class,
            class_config=class_config,
            child_configs=child_configs,
        )

        aci.get_diff(aci_class=aci_class)

        aci.post_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_registry_info
version_added: "3.19.0"
short_description: List ML Registries
description:
    - List ML Registries.
options:
    name:
        description:
            - Retrieve details about a specific ml registry.
        type: str
        required: false
    resource_group:
        description:
            - Name of a resource group to limit ml registries from.
              If not specified the subscription will be searched.
        required: false
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Get info on a specific ml regsitry
  azure.azcollection.azure_rm_ml_registry_info:
    name: my_registry
    resource_group: test_ml_registry
  register: result

- name: List all ml registries in subscription
  azure.azcollection.azure_rm_ml_registry_info:
  register: results
'''

RETURN = '''
ml_registries:
    description:
        - List of ML Registries.
    returned: always
    type: dict
    sample: [
        {
            "container_registry": {
                "acr_account_sku": "premium",
                "arm_resource_id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/azureml-rg-registry-xxxxxxx-xxx_xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.ContainerRegistry/registries/xxxxxxxxxxx"
            },
            "discovery_url": "https://eastus.api.azureml.ms/registrymanagement/v1.0/registries/registry-xxxxxxx-xxx/discovery",
            "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-registry/providers/Microsoft.MachineLearningServices/registries/registry-xxxxxxx-xxx",
            "identity": {
                "principal_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "tenant_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "type": "system_assigned"
            },
            "location": "eastus",
            "managed_resource_group": "{'additional_properties': {}, 'resource_id': '/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/azureml-rg-registry-xxxxxxx-xxx_xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx'}",
            "mlflow_registry_uri": "azureml://eastus.api.azureml.ms/mlflow/v1.0/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-registry/providers/Microsoft.MachineLearningServices/registries/registry-xxxxxxx-xxx",
            "name": "registry-xxxxxxx-xxx",
            "public_network_access": "Disabled",
            "replication_locations": [
                {
                    "location": "eastus",
                    "storage_config": {
                        "arm_resource_id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/azureml-rg-registry-xxxxxxx-xxx_xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.Storage/storageAccounts/xxxxxxxxxxx",
                        "replication_count": 1,
                        "storage_account_hns": false,
                        "storage_account_type": "standard_lrs"
                    }
                }
            ],
            "tags": {
                "foo": "bar"
            }
        }
    ]
'''  # NOQA


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.constants._common import Scope
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLRegistryInfo(MLClientCommon):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False
            ),
            resource_group=dict(
                type='str',
                required=False
            ),
        )

        self._client = None
        self.ml_workspace = None
        self.ml_registry = None

        required_by = {'name': 'resource_group'}

        self.results = dict(
            ml_registries=[]
        )

        super(AzureRMMLRegistryInfo, self).__init__(self.module_arg_spec,
                                                    supports_tags=False,
                                                    supports_check_mode=True,
                                                    required_by=required_by,
                                                    facts_module=True,
                                                    )

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name:
            try:
                result = self.client.registries.get(self.name)
                ml_registries = [self.entity_to_dict(result)]
            except ResourceNotFoundError:
                ml_registries = []
        else:
            scope = Scope.RESOURCE_GROUP if self.resource_group else Scope.SUBSCRIPTION
            results = self.client.registries.list(scope=scope)
            ml_registries = [self.entity_to_dict(x) for x in results]

        self.results['ml_registries'] = ml_registries

        return self.results


def main():
    AzureRMMLRegistryInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_memory_persistent_policy
short_description: Memory Persistent Policy configuration for Cisco Intersight
description:
  - Manages Memory Persistent Policy configuration on Cisco Intersight.
  - Configure Persistent Memory Modules (PMM) on servers including security, goals, and namespaces.
  - Supports both Intersight-managed and Operating System-managed configuration modes.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/memory/PersistentMemoryPolicy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Memory Persistent Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Memory Persistent Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  management_mode:
    description:
      - Configuration management mode for Persistent Memory.
      - C(configured-from-intersight) - Policy settings are configured and managed from Intersight.
      - C(configured-from-operating-system) - Associated servers are managed by Operating System tools and policy settings are unavailable.
      - When set to C(configured-from-operating-system), no other configuration fields are required or used.
    type: str
    choices: [configured-from-intersight, configured-from-operating-system]
    default: configured-from-intersight
  enable_security_passphrase:
    description:
      - Enable secure passphrase for Persistent Memory Modules.
      - When enabled, requires secure_passphrase parameter.
      - Only applicable when management_mode is C(configured-from-intersight).
    type: bool
    default: false
  secure_passphrase:
    description:
      - Secure passphrase to be applied on the Persistent Memory Modules on the server.
      - Required when enable_security_passphrase is true.
      - The allowed characters are a-z, A-Z, 0-9, and special characters =, !, &, #, $, %, +, ^, @, _, *, -.
      - Only applicable when management_mode is C(configured-from-intersight).
    type: str
  enable_goal:
    description:
      - Enable goal configuration for Persistent Memory Modules.
      - The Goal configured will be applicable to all the Persistent Memory Modules.
      - Goal modification will delete all existing regions and namespaces along with their data during profile deployment.
      - New regions and namespaces will be created after goal modification.
      - Only applicable when management_mode is C(configured-from-intersight).
    type: bool
    default: true
  memory_mode_percentage:
    description:
      - Volatile memory percentage for Memory Mode.
      - Valid range is 0-100.
      - Only applicable when enable_goal is true and management_mode is C(configured-from-intersight).
    type: int
    default: 0
  persistent_memory_type:
    description:
      - Type of Persistent Memory configuration.
      - C(app-direct) - Persistent Memory Modules are combined in an interleaved set.
      - C(app-direct-non-interleaved) - Persistent Memory Modules are not interleaved.
      - Only applicable when enable_goal is true and management_mode is C(configured-from-intersight).
    type: str
    choices: [app-direct, app-direct-non-interleaved]
    default: app-direct
  retain_namespaces:
    description:
      - Retain existing Persistent Memory Namespaces.
      - If false, all existing namespaces not listed in the namespaces parameter will be deleted along with their data.
      - Only applicable when management_mode is C(configured-from-intersight).
    type: bool
    default: true
  namespaces:
    description:
      - List of Logical Namespaces to be created or modified on the server.
      - Only applicable when management_mode is C(configured-from-intersight).
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Name of this Namespace to be created on the server.
        type: str
        required: true
      socket_id:
        description:
          - Socket ID of the region on which this Namespace has to be created or modified.
          - Valid values are 1, 2, 3, or 4.
        type: int
        choices: [1, 2, 3, 4]
        default: 1
      socket_memory_id:
        description:
          - Socket Memory ID of the region on which this Namespace has to be created or modified.
          - Only applicable when persistent_memory_type is C(app-direct-non-interleaved).
          - Valid values are 2, 4, 6, 8, 10, or 12.
        type: int
        choices: [2, 4, 6, 8, 10, 12]
        default: 2
      capacity:
        description:
          - Capacity of this Namespace in GiB.
          - Valid range is 1 to 9223372036854775807.
        type: int
        required: true
      mode:
        description:
          - Mode of this Namespace.
        type: str
        choices: [raw, block]
        default: raw
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create Memory Persistent Policy with Intersight management
  cisco.intersight.intersight_memory_persistent_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "pmem-policy-01"
    description: "Persistent Memory policy with security and namespaces"
    management_mode: configured-from-intersight
    enable_security_passphrase: true
    secure_passphrase: "SecurePass123!"
    enable_goal: true
    memory_mode_percentage: 0
    persistent_memory_type: app-direct
    retain_namespaces: true
    namespaces:
      - name: "ns1"
        socket_id: 1
        capacity: 100000
        mode: raw
      - name: "ns2"
        socket_id: 4
        capacity: 1000065
        mode: block
    state: present

- name: Create Memory Persistent Policy with non-interleaved configuration
  cisco.intersight.intersight_memory_persistent_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "pmem-policy-non-interleaved"
    description: "Non-interleaved persistent memory configuration"
    management_mode: configured-from-intersight
    enable_security_passphrase: true
    secure_passphrase: "MySecurePass123"
    enable_goal: true
    memory_mode_percentage: 0
    persistent_memory_type: app-direct-non-interleaved
    retain_namespaces: true
    namespaces:
      - name: "ns1"
        socket_id: 1
        socket_memory_id: 2
        capacity: 100000
        mode: raw
      - name: "ns2"
        socket_id: 2
        socket_memory_id: 4
        capacity: 200000
        mode: block
    state: present

- name: Create Memory Persistent Policy with OS management
  cisco.intersight.intersight_memory_persistent_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "pmem-policy-os-managed"
    description: "OS-managed persistent memory configuration"
    management_mode: configured-from-operating-system
    state: present

- name: Create Memory Persistent Policy without security passphrase
  cisco.intersight.intersight_memory_persistent_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "pmem-policy-no-security"
    description: "Policy without security passphrase"
    management_mode: configured-from-intersight
    enable_security_passphrase: false
    enable_goal: true
    memory_mode_percentage: 10
    persistent_memory_type: app-direct
    state: present

- name: Delete Memory Persistent Policy
  cisco.intersight.intersight_memory_persistent_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "pmem-policy-01"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "pmem-policy-01",
        "ObjectType": "memory.PersistentMemoryPolicy",
        "ManagementMode": "configured-from-intersight",
        "LocalSecurity": {
            "Enabled": true
        },
        "Goals": [
            {
                "MemoryModePercentage": 0,
                "PersistentMemoryType": "app-direct",
                "SocketId": "All Sockets"
            }
        ],
        "RetainNamespaces": true,
        "LogicalNamespaces": [
            {
                "Name": "ns1",
                "SocketId": 1,
                "SocketMemoryId": "Not Applicable",
                "Capacity": 100000,
                "Mode": "raw"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_namespace_capacity(capacity):
    """
    Validate namespace capacity is within acceptable range.
    """
    if capacity < 1 or capacity > 9223372036854775807:
        raise ValueError(f"Namespace capacity {capacity} is out of valid range (1-9223372036854775807 GiB)")
    return True


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        management_mode=dict(type='str', choices=['configured-from-intersight', 'configured-from-operating-system'], default='configured-from-intersight'),
        enable_security_passphrase=dict(type='bool', default=False),
        secure_passphrase=dict(type='str', no_log=True),
        enable_goal=dict(type='bool', default=True),
        memory_mode_percentage=dict(type='int', default=0),
        persistent_memory_type=dict(type='str', choices=['app-direct', 'app-direct-non-interleaved'], default='app-direct'),
        retain_namespaces=dict(type='bool', default=True),
        namespaces=dict(type='list', elements='dict', options=dict(
            name=dict(type='str', required=True),
            socket_id=dict(type='int', choices=[1, 2, 3, 4], default=1),
            socket_memory_id=dict(type='int', choices=[2, 4, 6, 8, 10, 12], default=2),
            capacity=dict(type='int', required=True),
            mode=dict(type='str', choices=['raw', 'block'], default='raw')
        ))
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
        required_if=[
            ['enable_security_passphrase', True, ['secure_passphrase']],
        ],
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path for Memory Persistent Policy
    resource_path = '/memory/PersistentMemoryPolicies'

    # Build base API body
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

        # Convert management mode to API format
        management_mode_value = intersight.module.params['management_mode']
        intersight.api_body['ManagementMode'] = management_mode_value

        # Only add configuration fields if managed from Intersight
        if management_mode_value == 'configured-from-intersight':
            # Add security configuration if enabled
            if intersight.module.params['enable_security_passphrase']:
                intersight.api_body['LocalSecurity'] = {
                    'Enabled': True,
                    'SecurePassphrase': intersight.module.params['secure_passphrase']
                }
            else:
                intersight.api_body['LocalSecurity'] = {
                    'Enabled': False
                }

            # Add goal configuration if enabled
            if intersight.module.params['enable_goal']:
                # Convert persistent memory type to API format
                pmem_type_value = intersight.module.params['persistent_memory_type']
                intersight.api_body['Goals'] = [
                    {
                        'MemoryModePercentage': intersight.module.params['memory_mode_percentage'],
                        'PersistentMemoryType': pmem_type_value,
                        'SocketId': 'All Sockets'
                    }
                ]

            # Add retain namespaces setting
            intersight.api_body['RetainNamespaces'] = intersight.module.params['retain_namespaces']

            # Add logical namespaces if provided
            if intersight.module.params.get('namespaces'):
                logical_namespaces = []
                pmem_type = intersight.module.params['persistent_memory_type']
                for ns in intersight.module.params['namespaces']:
                    # Validate namespace capacity
                    try:
                        validate_namespace_capacity(ns['capacity'])
                    except ValueError as e:
                        module.fail_json(msg=str(e))

                    # Determine SocketMemoryId based on persistent memory type
                    if pmem_type == 'app-direct-non-interleaved':
                        socket_memory_id = str(ns.get('socket_memory_id', 2))
                    else:
                        socket_memory_id = 'Not Applicable'

                    namespace_entry = {
                        'Name': ns['name'],
                        'SocketId': ns['socket_id'],
                        'SocketMemoryId': socket_memory_id,
                        'Capacity': ns['capacity'],
                        'Mode': ns['mode']
                    }
                    logical_namespaces.append(namespace_entry)

                intersight.api_body['LogicalNamespaces'] = logical_namespaces

    # Configure the policy
    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2012, Mark Theunissen <mark.theunissen@gmail.com>
# Sponsored by Four Kitchens http://fourkitchens.com.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: mysql_user
short_description: Adds or removes a user from a MySQL or MariaDB database
description:
   - Adds or removes a user from a MySQL or MariaDB database.
options:
  name:
    description:
      - Name of the user (role) to add or remove.
    type: str
    required: true
  password:
    description:
      - Set the user's password. Only for C(mysql_native_password) authentication.
        For other authentication plugins see the combination of I(plugin), I(plugin_hash_string), I(plugin_auth_string).
    type: str
  encrypted:
    description:
      - Indicate that the 'password' field is a `mysql_native_password` hash.
    type: bool
    default: false
  host:
    description:
      - The 'host' part of the MySQL username.
    type: str
    default: localhost
  host_all:
    description:
      - Override the host option, making ansible apply changes
        to all hostnames for a given user.
      - This option cannot be used when creating users.
    type: bool
    default: false
  priv:
    description:
      - "MySQL privileges string in the format: C(db.table:priv1,priv2)."
      - Additionally, there must be no spaces between the table and the privilege as this will yield a non-idempotent check mode.
      - "Multiple privileges can be specified by separating each one using
        a forward slash: C(db.table1:priv/db.table2:priv)."
      - The format is based on MySQL C(GRANT) statement.
      - Database and table names can be quoted, MySQL-style.
      - If column privileges are used, the C(priv1,priv2) part must be
        exactly as returned by a C(SHOW GRANT) statement. If not followed,
        the module will always report changes. It includes grouping columns
        by permission (C(SELECT(col1,col2)) instead of C(SELECT(col1),SELECT(col2))).
      - Can be passed as a dictionary (see the examples).
      - Supports GRANTs for procedures and functions (see the examples).
      - "Note: If you pass the same C(db.table) combination to this parameter
        two or more times with different privileges,
        for example, C('*.*:SELECT/*.*:SHOW VIEW'), only the last one will be applied,
        in this example, it will be C(SHOW VIEW) respectively.
        Use C('*.*:SELECT,SHOW VIEW') instead to apply both."
    type: raw
  append_privs:
    description:
      - Append the privileges defined by priv to the existing ones for this
        user instead of overwriting existing ones. Mutually exclusive with I(subtract_privs).
    type: bool
    default: false
  subtract_privs:
    description:
      - Revoke the privileges defined by the I(priv) option and keep other existing privileges.
        If set, invalid privileges in I(priv) are ignored.
        Mutually exclusive with I(append_privs).
    version_added: '3.2.0'
    type: bool
    default: false
  tls_requires:
    description:
      - Set requirement for secure transport as a dictionary of requirements (see the examples).
      - Valid requirements are SSL, X509, SUBJECT, ISSUER, CIPHER.
      - SUBJECT, ISSUER and CIPHER are complementary, and mutually exclusive with SSL and X509.
      - U(https://mariadb.com/kb/en/securing-connections-for-client-and-server/#requiring-tls).
    type: dict
    version_added: 1.0.0
  sql_log_bin:
    description:
      - Whether binary logging should be enabled or disabled for the connection.
    type: bool
    default: true
  force_context:
    description:
      - Sets the С(mysql) system database as context for the executed statements (it will be used
        as a database to connect to). Useful if you use binlog / replication filters in MySQL as
        per default the statements can not be caught by a binlog / replication filter, they require
        a database to be set to work, otherwise the replication can break down.
      - See U(https://dev.mysql.com/doc/refman/8.0/en/replication-options-binary-log.html#option_mysqld_binlog-ignore-db)
        for a description on how binlog filters work (filtering on the primary).
      - See U(https://dev.mysql.com/doc/refman/8.0/en/replication-options-replica.html#option_mysqld_replicate-ignore-db)
        for a description on how replication filters work (filtering on the replica).
    type: bool
    default: false
    version_added: '3.1.0'
  state:
    description:
      - Whether the user should exist.
      - When C(absent), removes the user.
    type: str
    choices: [ absent, present ]
    default: present
  check_implicit_admin:
    description:
      - Check if mysql allows login as root/nopassword before trying supplied credentials.
      - If success, passed I(login_user)/I(login_password) will be ignored.
    type: bool
    default: false
  update_password:
    description:
      - C(always) will update passwords if they differ. This affects I(password) and the combination of I(plugin), I(plugin_hash_string), I(plugin_auth_string).
      - C(on_create) will only set the password or the combination of I(plugin), I(plugin_hash_string), I(plugin_auth_string) for newly created users.
      - "C(on_new_username) works like C(on_create), but it tries to reuse an existing password: If one different user
        with the same username exists, or multiple different users with the same username and equal C(plugin) and
        C(authentication_string) attribute, the existing C(plugin) and C(authentication_string) are used for the
        new user instead of the I(password), I(plugin), I(plugin_hash_string) or I(plugin_auth_string) argument."
    type: str
    choices: [ always, on_create, on_new_username ]
    default: always
  plugin:
    description:
      - User's plugin to authenticate (``CREATE USER user IDENTIFIED WITH plugin``).
    type: str
    version_added: '0.1.0'
  plugin_hash_string:
    description:
      - User's plugin hash string (``CREATE USER user IDENTIFIED WITH plugin AS plugin_hash_string``).
    type: str
    version_added: '0.1.0'
  plugin_auth_string:
    description:
      - User's plugin auth_string (``CREATE USER user IDENTIFIED WITH plugin BY plugin_auth_string``).
      - If I(plugin) is ``pam`` (MariaDB) or ``auth_pam`` (MySQL) an optional I(plugin_auth_string) can be used to choose a specific PAM service.
      - You need to define a I(salt) to have idempotence on password change with ``caching_sha2_password`` and ``sha256_password`` plugins.
    type: str
    version_added: '0.1.0'
  salt:
    description:
      - Salt used to generate password hash from I(plugin_auth_string).
      - Salt length must be 20 characters.
      - Salt only support ``caching_sha2_password`` or ``sha256_password`` authentication I(plugin).
    type: str
    version_added: '3.10.0'
  resource_limits:
    description:
      - Limit the user for certain server resources. Provided since MySQL 5.6 / MariaDB 10.2.
      - "Available options are C(MAX_QUERIES_PER_HOUR: num), C(MAX_UPDATES_PER_HOUR: num),
        C(MAX_CONNECTIONS_PER_HOUR: num), C(MAX_USER_CONNECTIONS: num), C(MAX_STATEMENT_TIME: num) (supported only for MariaDB since collection version 3.7.0)."
      - Used when I(state=present), ignored otherwise.
    type: dict
    version_added: '0.1.0'
  session_vars:
    description:
      - "Dictionary of session variables in form of C(variable: value) to set at the beginning of module execution."
      - Cannot be used to set global variables, use the M(ansible.mysql.mysql_variables) module instead.
    type: dict
    version_added: '3.6.0'
  password_expire:
    description:
      - C(never) - I(password) will never expire.
      - C(default) - I(password) is defined using global system variable I(default_password_lifetime) setting.
      - C(interval) - I(password) will expire in days which is defined in I(password_expire_interval).
      - C(now) - I(password) will expire immediately.
    type: str
    choices: [ now, never, default, interval ]
    version_added: '3.9.0'
  password_expire_interval:
    description:
      - Number of days I(password) will expire. Requires I(password_expire=interval).
    type: int
    version_added: '3.9.0'

  column_case_sensitive:
    description:
      - The default is C(true).
      - When C(true), the module will not uppercase the field names in the privileges.
      - When C(false), the field names will be upper-cased. This is the default
      - This feature was introduced because MySQL 8 and above uses case sensitive
        fields names in privileges.
      - The default changed from C(false) to C(true) in v4.0.0
    type: bool
    default: true
    version_added: '3.8.0'

  locked:
    description:
      - Lock account to prevent connections using it.
      - This is primarily used for creating a user that will act as a DEFINER on stored procedures.
      - If not specified leaves the lock state as is (for a new user creates unlocked).
    type: bool
    version_added: '3.13.0'

  attributes:
    description:
      - "Create, update, or delete user attributes (arbitrary 'key: value' comments) for the user."
      - MySQL server must support the INFORMATION_SCHEMA.USER_ATTRIBUTES table. Provided since MySQL 8.0.
      - To delete an existing attribute, set its value to null.
    type: dict
    version_added: '3.9.0'

notes:
   - Compatible with MySQL or MariaDB.
   - "MySQL server installs with default I(login_user) of C(root) and no password.
     To secure this user as part of an idempotent playbook, you must create at least two tasks:
     1) change the root user's password, without providing any I(login_user)/I(login_password) details,
     2) drop a C(~/.my.cnf) file containing the new root credentials.
     Subsequent runs of the playbook will then succeed by reading the new credentials from the file."
   - Currently, there is only support for the C(mysql_native_password) encrypted password hash module.

attributes:
  check_mode:
    support: full
  idempotent:
    support: full

seealso:
- module: ansible.mysql.mysql_info
- name: MySQL access control and account management reference
  description: Complete reference of the MySQL access control and account management documentation.
  link: https://dev.mysql.com/doc/refman/8.0/en/access-control.html
- name: MySQL provided privileges reference
  description: Complete reference of the MySQL provided privileges documentation.
  link: https://dev.mysql.com/doc/refman/8.0/en/privileges-provided.html

author:
- Jonathan Mainguy (@Jmainguy)
- Benjamin Malynovytch (@bmalynovytch)
- Lukasz Tomaszkiewicz (@tomaszkiewicz)
- kmarse (@kmarse)
- Laurent Indermühle (@laurent-indermuehle)
- E.S. Rosenberg (@Keeper-of-the-Keys)

extends_documentation_fragment:
- ansible.mysql.mysql
'''

EXAMPLES = r'''
# If you encounter the "Please explicitly state intended protocol" error,
# use the login_unix_socket argument
- name: Removes anonymous user account for localhost
  ansible.mysql.mysql_user:
    name: ''
    host: localhost
    state: absent
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Removes all anonymous user accounts
  ansible.mysql.mysql_user:
    name: ''
    host_all: true
    state: absent

- name: Create database user with name 'bob' and password '12345' with all database privileges
  ansible.mysql.mysql_user:
    name: bob
    password: 12345
    priv: '*.*:ALL'
    state: present

- name: Create database user using hashed password with all database privileges
  ansible.mysql.mysql_user:
    name: bob
    password: '*EE0D72C1085C46C5278932678FBE2C6A782821B4'
    encrypted: true
    priv: '*.*:ALL'
    state: present

# Set session var wsrep_on=off before creating the user
- name: Create database user with password and all database privileges and 'WITH GRANT OPTION'
  ansible.mysql.mysql_user:
    name: bob
    password: 12345
    priv: '*.*:ALL,GRANT'
    state: present
    session_vars:
      wsrep_on: 'off'

- name: Create user with password, all database privileges and 'WITH GRANT OPTION' in db1 and db2
  ansible.mysql.mysql_user:
    state: present
    name: bob
    password: 12345dd
    priv:
      'db1.*': 'ALL,GRANT'
      'db2.*': 'ALL,GRANT'

# Use 'PROCEDURE' instead of 'FUNCTION' to apply GRANTs for a MySQL procedure instead.
- name: Grant a user the right to execute a function
  ansible.mysql.mysql_user:
    name: readonly
    password: 12345
    priv:
      FUNCTION my_db.my_function: EXECUTE
    state: present

- name: Modify user attributes, creating the attribute 'foo' and removing the attribute 'bar'
  ansible.mysql.mysql_user:
    name: bob
    attributes:
      foo: "foo"
      bar: null

- name: Modify user to require TLS connection with a valid client certificate
  ansible.mysql.mysql_user:
    name: bob
    tls_requires:
      x509:
    state: present

- name: Modify user to require TLS connection with a specific client certificate and cipher
  ansible.mysql.mysql_user:
    name: bob
    tls_requires:
      subject: '/CN=alice/O=MyDom, Inc./C=US/ST=Oregon/L=Portland'
      cipher: 'ECDHE-ECDSA-AES256-SHA384'

- name: Modify user to no longer require SSL
  ansible.mysql.mysql_user:
    name: bob
    tls_requires:

- name: Ensure no user named 'sally'@'localhost' exists, also passing in the auth credentials
  ansible.mysql.mysql_user:
    login_user: root
    login_password: 123456
    name: sally
    state: absent

# check_implicit_admin example
- name: >
    Ensure no user named 'sally'@'localhost' exists, also passing in the auth credentials.
    If mysql allows root/nopassword login, try it without the credentials first.
    If it's not allowed, pass the credentials
  ansible.mysql.mysql_user:
    check_implicit_admin: true
    login_user: root
    login_password: 123456
    name: sally
    state: absent

- name: Ensure no user named 'sally' exists at all
  ansible.mysql.mysql_user:
    name: sally
    host_all: true
    state: absent

- name: Specify grants composed of more than one word
  ansible.mysql.mysql_user:
    name: replication
    password: 12345
    priv: "*.*:REPLICATION CLIENT"
    state: present

- name: Revoke all privileges for user 'bob' and password '12345'
  ansible.mysql.mysql_user:
    name: bob
    password: 12345
    priv: "*.*:USAGE"
    state: present

# Example privileges string format
# mydb.*:INSERT,UPDATE/anotherdb.*:SELECT/yetanotherdb.*:ALL

- name: Example using login_unix_socket to connect to server
  ansible.mysql.mysql_user:
    name: root
    password: abc123
    login_unix_socket: /var/run/mysqld/mysqld.sock

- name: Example of skipping binary logging while adding user 'bob'
  ansible.mysql.mysql_user:
    name: bob
    password: 12345
    priv: "*.*:USAGE"
    state: present
    sql_log_bin: false

- name: Create user 'bob' authenticated with plugin 'AWSAuthenticationPlugin'
  ansible.mysql.mysql_user:
    name: bob
    plugin: AWSAuthenticationPlugin
    plugin_hash_string: RDS
    priv: '*.*:ALL'
    state: present

- name: Create user 'bob' authenticated with plugin 'caching_sha2_password' and static salt
  ansible.mysql.mysql_user:
    name: bob
    plugin: caching_sha2_password
    plugin_auth_string: password
    salt: 1234567890abcdefghij

- name: Limit bob's resources to 10 queries per hour and 5 connections per hour
  ansible.mysql.mysql_user:
    name: bob
    resource_limits:
      MAX_QUERIES_PER_HOUR: 10
      MAX_CONNECTIONS_PER_HOUR: 5

- name: Ensure bob does not have the DELETE privilege
  ansible.mysql.mysql_user:
    name: bob
    subtract_privs: true
    priv:
      'db1.*': DELETE

- name: Create locked user to act as a definer on procedures
  ansible.mysql.mysql_user:
    name: readonly_procedures_locked
    locked: true
    priv:
      db1.*: SELECT

# Example .my.cnf file for setting the root password
# [client]
# user=root
# password=n<_665{vS43y
'''

RETURN = '''#'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.mysql.plugins.module_utils.database import SQLParseError
from ansible_collections.ansible.mysql.plugins.module_utils.mysql import (
    mysql_connect,
    mysql_driver,
    mysql_driver_fail_msg,
    mysql_common_argument_spec,
    set_session_vars,
)
from ansible_collections.ansible.mysql.plugins.module_utils.user import (
    convert_priv_dict_to_str,
    get_mode,
    InvalidPrivsError,
    limit_resources,
    privileges_unpack,
    sanitize_requires,
    user_add,
    user_delete,
    user_exists,
    user_mod,
)
from ansible.module_utils.common.text.converters import to_native


# ===========================================
# Module execution.
#


def main():
    argument_spec = mysql_common_argument_spec()
    argument_spec.update(
        name=dict(type='str', required=True),
        password=dict(type='str', no_log=True),
        encrypted=dict(type='bool', default=False),
        host=dict(type='str', default='localhost'),
        host_all=dict(type="bool", default=False),
        state=dict(type='str', default='present', choices=['absent', 'present']),
        priv=dict(type='raw'),
        tls_requires=dict(type='dict'),
        append_privs=dict(type='bool', default=False),
        subtract_privs=dict(type='bool', default=False),
        attributes=dict(type='dict'),
        check_implicit_admin=dict(type='bool', default=False),
        update_password=dict(type='str', default='always', choices=['always', 'on_create', 'on_new_username'], no_log=False),
        sql_log_bin=dict(type='bool', default=True),
        plugin=dict(default=None, type='str'),
        plugin_hash_string=dict(default=None, type='str'),
        plugin_auth_string=dict(default=None, type='str'),
        salt=dict(default=None, type='str'),
        resource_limits=dict(type='dict'),
        force_context=dict(type='bool', default=False),
        session_vars=dict(type='dict'),
        column_case_sensitive=dict(type='bool', default=True),
        password_expire=dict(type='str', choices=['now', 'never', 'default', 'interval'], no_log=True),
        password_expire_interval=dict(type='int', required_if=[('password_expire', 'interval', True)], no_log=True),
        locked=dict(type='bool'),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=(('append_privs', 'subtract_privs'),)
    )
    login_user = module.params["login_user"]
    login_password = module.params["login_password"]
    user = module.params["name"]
    password = module.params["password"]
    encrypted = module.boolean(module.params["encrypted"])
    host = module.params["host"].lower()
    host_all = module.params["host_all"]
    state = module.params["state"]
    priv = module.params["priv"]
    tls_requires = sanitize_requires(module.params["tls_requires"])
    check_implicit_admin = module.params["check_implicit_admin"]
    connect_timeout = module.params["connect_timeout"]
    config_file = module.params["config_file"]
    append_privs = module.boolean(module.params["append_privs"])
    subtract_privs = module.boolean(module.params['subtract_privs'])
    update_password = module.params['update_password']
    attributes = module.params['attributes']
    ssl_cert = module.params["client_cert"]
    ssl_key = module.params["client_key"]
    ssl_ca = module.params["ca_cert"]
    check_hostname = module.params["check_hostname"]
    db = ''
    if module.params["force_context"]:
        db = 'mysql'
    sql_log_bin = module.params["sql_log_bin"]
    plugin = module.params["plugin"]
    plugin_hash_string = module.params["plugin_hash_string"]
    plugin_auth_string = module.params["plugin_auth_string"]
    salt = module.params["salt"]
    resource_limits = module.params["resource_limits"]
    session_vars = module.params["session_vars"]
    column_case_sensitive = module.params["column_case_sensitive"]
    password_expire = module.params["password_expire"]
    password_expire_interval = module.params["password_expire_interval"]
    locked = module.boolean(module.params['locked'])

    if priv and not isinstance(priv, (str, dict)):
        module.fail_json(msg="priv parameter must be str or dict but %s was passed" % type(priv))

    if priv and isinstance(priv, dict):
        priv = convert_priv_dict_to_str(priv)

    if mysql_driver is None:
        module.fail_json(msg=mysql_driver_fail_msg)

    if password_expire_interval and password_expire_interval < 1:
        module.fail_json(msg="password_expire_interval value \
                             should be positive number")

    if salt:
        if not plugin_auth_string:
            module.fail_json(msg="salt requires plugin_auth_string")
        if len(salt) != 20:
            module.fail_json(msg="salt must be 20 characters long")
        if plugin not in ['caching_sha2_password', 'sha256_password']:
            module.fail_json(msg="salt requires caching_sha2_password or sha256_password plugin")

    cursor = None
    try:
        if check_implicit_admin:
            try:
                cursor, db_conn = mysql_connect(module, "root", "", config_file, ssl_cert, ssl_key, ssl_ca, db,
                                                connect_timeout=connect_timeout, check_hostname=check_hostname, autocommit=True)
            except Exception:
                pass

        if not cursor:
            cursor, db_conn = mysql_connect(module, login_user, login_password, config_file, ssl_cert, ssl_key, ssl_ca, db,
                                            connect_timeout=connect_timeout, check_hostname=check_hostname, autocommit=True)
    except Exception as e:
        module.fail_json(msg="unable to connect to database, check login_user and login_password are correct or %s has the credentials. "
                             "Exception message: %s" % (config_file, to_native(e)))

    if not sql_log_bin:
        cursor.execute("SET SQL_LOG_BIN=0;")

    if session_vars:
        set_session_vars(module, cursor, session_vars)

    if priv is not None:
        try:
            mode = get_mode(cursor)
        except Exception as e:
            module.fail_json(msg=to_native(e))

        priv = privileges_unpack(priv, mode, column_case_sensitive, ensure_usage=not subtract_privs)
    password_changed = False
    final_attributes = None
    if state == "present":
        if user_exists(cursor, user, host, host_all):
            try:
                if update_password == "always":
                    result = user_mod(cursor, user, host, host_all, password, encrypted,
                                      plugin, plugin_hash_string, plugin_auth_string, salt,
                                      priv, append_privs, subtract_privs, attributes, tls_requires, module,
                                      password_expire, password_expire_interval, locked=locked)

                else:
                    result = user_mod(cursor=cursor, user=user, host=host, host_all=host_all, password=None,
                                      encrypted=encrypted, plugin=None, plugin_hash_string=None, plugin_auth_string=None,
                                      salt=None, new_priv=priv, append_privs=append_privs, subtract_privs=subtract_privs,
                                      attributes=attributes, tls_requires=tls_requires, module=module,
                                      password_expire=password_expire, password_expire_interval=password_expire_interval,
                                      locked=locked)
                changed = result['changed']
                msg = result['msg']
                password_changed = result['password_changed']
                final_attributes = result['attributes']

            except (SQLParseError, InvalidPrivsError, mysql_driver.Error) as e:
                module.fail_json(msg=to_native(e))
        else:
            if host_all:
                module.fail_json(msg="host_all parameter cannot be used when adding a user")
            try:
                if subtract_privs:
                    priv = None  # avoid granting unwanted privileges
                reuse_existing_password = update_password == 'on_new_username'
                result = user_add(cursor, user, host, host_all, password, encrypted,
                                  plugin, plugin_hash_string, plugin_auth_string, salt,
                                  priv, attributes, tls_requires, reuse_existing_password, module,
                                  password_expire, password_expire_interval, locked=locked)
                changed = result['changed']
                password_changed = result['password_changed']
                final_attributes = result['attributes']
                if changed:
                    msg = "User added"

            except (SQLParseError, InvalidPrivsError, mysql_driver.Error) as e:
                module.fail_json(msg=to_native(e))

        if resource_limits:
            changed = limit_resources(module, cursor, user, host, resource_limits, module.check_mode) or changed

    elif state == "absent":
        if user_exists(cursor, user, host, host_all):
            changed = user_delete(cursor, user, host, host_all, module.check_mode)
            msg = "User deleted"
        else:
            changed = False
            msg = "User doesn't exist"
    module.exit_json(changed=changed, user=user, msg=msg, password_changed=password_changed, attributes=final_attributes)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2023, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_image_push

short_description: Push Docker images to registries

version_added: 3.6.0

description:
  - Pushes a Docker image to a registry.
extends_documentation_fragment:
  - community.docker._docker.api_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker

attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
  idempotent:
    support: full

options:
  name:
    description:
      - Image name. Name format must be one of V(name), V(repository/name), or V(registry_server:port/name).
      - The name can optionally include the tag by appending V(:tag_name), or it can contain a digest by appending V(@hash:digest).
    type: str
    required: true
  tag:
    description:
      - Select which image to push. Defaults to V(latest).
      - If O(name) parameter format is C(name:tag) or C(image@hash:digest), then O(tag) will be ignored.
    type: str
    default: latest

requirements:
  - "Docker API >= 1.25"

author:
  - Felix Fontein (@felixfontein)

seealso:
  - module: community.docker.docker_image_pull
  - module: community.docker.docker_image_remove
  - module: community.docker.docker_image_tag
"""

EXAMPLES = r"""
---
- name: Push an image
  community.docker.docker_image_push:
    name: registry.example.com:5000/repo/image
    tag: latest
"""

RETURN = r"""
image:
  description: Image inspection results for the affected image.
  returned: success
  type: dict
  sample: {}
"""

import base64
import traceback
import typing as t

from ansible_collections.community.docker.plugins.module_utils._api.auth import (
    get_config_header,
    resolve_repository_name,
)
from ansible_collections.community.docker.plugins.module_utils._api.errors import (
    DockerException,
)
from ansible_collections.community.docker.plugins.module_utils._api.utils.utils import (
    parse_repository_tag,
)
from ansible_collections.community.docker.plugins.module_utils._common_api import (
    AnsibleDockerClient,
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils._util import (
    DockerBaseClass,
    is_image_name_id,
    is_valid_tag,
)


class ImagePusher(DockerBaseClass):
    def __init__(self, client: AnsibleDockerClient) -> None:
        super().__init__()

        self.client = client
        self.check_mode = self.client.check_mode

        parameters = self.client.module.params
        self.name: str = parameters["name"]
        self.tag: str = parameters["tag"]

        if is_image_name_id(self.name):
            self.client.fail("Cannot push an image by ID")
        if not is_valid_tag(self.tag, allow_empty=True):
            self.client.fail(f'"{self.tag}" is not a valid docker tag!')

        # If name contains a tag, it takes precedence over tag parameter.
        repo, repo_tag = parse_repository_tag(self.name)
        if repo_tag:
            self.name = repo
            self.tag = repo_tag

        if is_image_name_id(self.tag):
            self.client.fail("Cannot push an image by digest")
        if not is_valid_tag(self.tag, allow_empty=False):
            self.client.fail(f'"{self.tag}" is not a valid docker tag!')

    def push(self) -> dict[str, t.Any]:
        image = self.client.find_image(name=self.name, tag=self.tag)
        if not image:
            self.client.fail(f"Cannot find image {self.name}:{self.tag}")

        actions: list[str] = []
        results: dict[str, t.Any] = {
            "changed": False,
            "actions": actions,
            "image": image,
        }

        push_registry, push_repo = resolve_repository_name(self.name)
        try:
            actions.append(f"Pushed image {self.name}:{self.tag}")

            headers = {}
            header = get_config_header(self.client, push_registry)
            if not header:
                # For some reason, from Docker 28.3.3 on not specifying X-Registry-Auth seems to be invalid.
                # See https://github.com/moby/moby/issues/50614.
                header = base64.urlsafe_b64encode(b"{}")
            headers["X-Registry-Auth"] = header
            response = self.client._post_json(
                self.client._url("/images/{0}/push", self.name),
                data=None,
                headers=headers,
                stream=True,
                params={"tag": self.tag},
            )
            self.client._raise_for_status(response)
            for line in self.client._stream_helper(response, decode=True):
                self.log(line, pretty_print=True)
                if line.get("errorDetail"):
                    raise RuntimeError(line["errorDetail"]["message"])
                status = line.get("status")
                if status in ("Pushing", "Pushed"):
                    results["changed"] = True
        except Exception as exc:  # pylint: disable=broad-exception-caught
            if "unauthorized" in str(exc):
                if "authentication required" in str(exc):
                    self.client.fail(
                        f"Error pushing image {push_registry}/{push_repo}:{self.tag} - {exc}. Try logging into {push_registry} first."
                    )
                else:
                    self.client.fail(
                        f"Error pushing image {push_registry}/{push_repo}:{self.tag} - {exc}. Does the repository exist?"
                    )
            self.client.fail(f"Error pushing image {self.name}:{self.tag}: {exc}")

        return results


def main() -> None:
    argument_spec = {
        "name": {"type": "str", "required": True},
        "tag": {"type": "str", "default": "latest"},
    }

    client = AnsibleDockerClient(
        argument_spec=argument_spec,
        supports_check_mode=False,
    )

    try:
        results = ImagePusher(client).push()
        client.module.exit_json(**results)
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when trying to talk to the Docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: mq_broker_config
version_added: 6.0.0
short_description: Update Amazon MQ broker configuration
description:
  - Update configuration for an MQ broker.
  - If new configuration differs from the current one a new configuration
    is created and the new version is assigned to the broker.
  - Optionally allows broker reboot to make changes effective immediately.
author:
  - FCO (@fotto)
options:
  broker_id:
    description:
      - The ID of the MQ broker to work on.
    type: str
    required: true
  config_xml:
    description:
      - The maximum number of results to return.
    type: str
    required: true
  config_description:
    description:
      - Description to set on new configuration revision.
    type: str
  reboot:
    description:
      - Reboot broker after new config has been applied.
    type: bool
    default: false
extends_documentation_fragment:
  - amazon.aws.boto3
  - amazon.aws.common.modules
  - amazon.aws.region.modules
"""

EXAMPLES = r"""
- name: send new XML config to broker relying on credentials from environment
  community.aws.mq_broker_config:
    broker_id: "aws-mq-broker-id"
    config_xml: "{{ lookup('file', 'activemq.xml' )}}"
    region: "{{ aws_region }}"

- name: send new XML config to broker and reboot if necessary
  community.aws.mq_broker_config:
    broker_id: "aws-mq-broker-id"
    config_xml: "{{ lookup('file', 'activemq2.xml' )}}"
    reboot: true

- name: send new broker config and set all credentials explicitly
  community.aws.mq_broker_config:
    broker_id: "{{ broker_id }}"
    config_xml: "{{ lookup('file', 'activemq3.xml')}}"
    config_description: "custom description for configuration object"
    register: result
"""

RETURN = r"""
broker:
  description: API response of describe_broker() converted to snake yaml after changes have been applied.
  type: dict
  returned: success
configuration:
  description: Details about new configuration object.
  returned: I(changed=true)
  type: complex
  contains:
    id:
      description: Configuration ID of broker configuration.
      type: str
      example: c-386541b8-3139-42c2-9c2c-a4c267c1714f
    revision:
      description: Revision of the configuration that will be active after next reboot.
      type: int
      example: 4
"""

import base64
import re

try:
    import botocore
except ImportError:
    # handled by AnsibleAWSModule
    pass

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule

DEFAULTS = {"reboot": False}
FULL_DEBUG = False


def is_same_config(old, new):
    # we a simple comparision here: strip down spaces and compare the rest
    # TODO: use same XML normalizer on new as used by AWS before comparing strings
    old_stripped = re.sub(r"\s+", " ", old, flags=re.S).rstrip()
    new_stripped = re.sub(r"\s+", " ", new, flags=re.S).rstrip()
    return old_stripped == new_stripped


def get_broker_info(conn, module):
    try:
        return conn.describe_broker(BrokerId=module.params["broker_id"])
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        if module.check_mode:
            return {
                "broker_id": module.params["broker_id"],
            }
        module.fail_json_aws(e, msg="Couldn't get broker details.")


def get_current_configuration(conn, module, cfg_id, cfg_revision):
    try:
        return conn.describe_configuration_revision(ConfigurationId=cfg_id, ConfigurationRevision=str(cfg_revision))
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't get configuration revision.")


def create_and_assign_config(conn, module, broker_id, cfg_id, cfg_xml_encoded):
    kwargs = {"ConfigurationId": cfg_id, "Data": cfg_xml_encoded}
    if "config_description" in module.params and module.params["config_description"]:
        kwargs["Description"] = module.params["config_description"]
    else:
        kwargs["Description"] = "Updated through community.aws.mq_broker_config ansible module"
    #
    try:
        c_response = conn.update_configuration(**kwargs)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't create new configuration revision.")
    #
    new_config_revision = c_response["LatestRevision"]["Revision"]
    try:
        b_response = conn.update_broker(
            BrokerId=broker_id, Configuration={"Id": cfg_id, "Revision": new_config_revision}
        )
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't assign new configuration revision to broker.")
    #
    return (c_response, b_response)


def reboot_broker(conn, module, broker_id):
    try:
        return conn.reboot_broker(BrokerId=broker_id)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't reboot broker.")


def ensure_config(conn, module):
    broker_id = module.params["broker_id"]
    broker_info = get_broker_info(conn, module)
    changed = False
    if module.check_mode and "Configurations" not in broker_info:
        # not result from get_broker_info(). use requeste config
        current_cfg_decoded = module.params["config_xml"]
    else:
        current_cfg = broker_info["Configurations"]["Current"]
        if "Pending" in broker_info["Configurations"]:
            current_cfg = broker_info["Configurations"]["Pending"]
        current_cfg_encoded = get_current_configuration(conn, module, current_cfg["Id"], current_cfg["Revision"])[
            "Data"
        ]
        current_cfg_decoded = base64.b64decode(current_cfg_encoded.encode()).decode()

    if is_same_config(current_cfg_decoded, module.params["config_xml"]):
        return {"changed": changed, "broker": camel_dict_to_snake_dict(broker_info, ignore_list=["Tags"])}

    (c_response, _b_response) = (None, None)
    if not module.check_mode:
        new_cfg_encoded = base64.b64encode(module.params["config_xml"].encode()).decode()
        (c_response, _b_response) = create_and_assign_config(
            conn, module, broker_id, current_cfg["Id"], new_cfg_encoded
        )
    #
    changed = True

    if changed and module.params["reboot"] and not module.check_mode:
        reboot_broker(conn, module, broker_id)
    #
    broker_info = get_broker_info(conn, module)
    return_struct = {
        "changed": changed,
        "broker": camel_dict_to_snake_dict(broker_info, ignore_list=["Tags"]),
        "configuration": {"id": c_response["Id"], "revision": c_response["LatestRevision"]["Revision"]},
    }
    if FULL_DEBUG:
        return_struct["old_config_xml"] = base64.b64decode(current_cfg_encoded)
        return_struct["new_config_xml"] = module.params["config_xml"]
        return_struct["old_config_revision"] = current_cfg["Revision"]
    return return_struct


def main():
    argument_spec = dict(
        broker_id=dict(required=True, type="str"),
        config_xml=dict(required=True, type="str"),
        config_description=dict(required=False, type="str"),
        reboot=dict(required=False, type="bool", default=DEFAULTS["reboot"]),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    connection = module.client("mq")

    try:
        result = ensure_config(connection, module)
    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_eventgrid_topic_info
version_added: "3.13.0"
short_description: List Eventgrid Topics
description:
    - List Eventgrid Topics.
options:
    name:
        description:
            - Specify the name of the Eventgrid topic.
        required: false
        type: str
    resource_group:
        description:
            - Name of a resource group to limit topics from.  If not specified the subscription
              will be searched.
        required: false
        type: str
    odata_query:
        description:
            - The query used to filter the search results using OData syntax.
        required: false
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Get info on a Topic
  azure.azcollection.azure_rm_eventgrid_topic_info:
    resource_group: rg1
    odata_query: "contains(name, 'PATTERN')"
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: true
eventgrid_topics:
    description:
        - List of Eventgrid Topics.
    returned: always
    type: complex
    contains:
        data_residency_boundary:
            description:
                - Residncy boundary
            returned: always
            type: str
            sample: WithinGeopair
        disable_local_auth:
            description:
                - Whether local auth is disabled
            returned: always
            type: bool
            sample: false
        endpoint:
            description:
                - Rest API endpoint to use to post events
            returned: always
            type: str
            sample: https://myTopic-20070.westus3-1.eventgrid.azure.net/api/events
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.EventGrid/topics/myTopic
        inbound_ip_rules:
            description:
                - Rules for network access
            type: complex
            returned: when-enabled
            contains:
                action:
                    description:
                        - What action will be taken, Allow or Disallow
                    returned: always
                    type: str
                    sample: Allow
                ip_mask:
                    description:
                        - CIDR notation of the network to act on.
                    returned: always
                    type: str
                    sample: 10.0.0.0/8
        input_schema:
            description:
                - The input schema assigned to this Topic
            returned: always
            type: str
            sample: EventGridSchema
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: westus3
        metric_resource_id:
            description:
                - Metric Resource ID.
            returned: always
            type: str
            sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        minimum_tls_version_allowed:
            description:
                - The minimum TLS version allowed
            returned: always
            type: str
            sample: 1.0
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: myTopic
        provisioning_state:
            description:
                - The provisioning state.
            returned: always
            type: str
            sample: Succeeded
        public_network_access:
            description:
                - Indicates if boot diagnostics are enabled.
            returned: always
            type: bool
            sample: true
        topic_keys:
            description:
                - Shared access keys to a topic
            type: complex
            returned: always
            contains:
                key1:
                    description:
                        - Shared access key 1
                    type: str
                    returned: always
                    sample: FYKuIOq0xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxZEGV3gr
                key2:
                    description:
                        - Shared access key 2
                    type: str
                    returned: always
                    sample: D8QRdfVrxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxZEGdMMc
        type:
            description:
                - Type of Eventgrid
            type: str
            returned: always
            sample: Microsoft.EventGrid/topics
'''


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.mgmt.eventgrid import EventGridManagementClient
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMEventgridTopic(AzureRMModuleBase):

    @property
    def client(self):
        self.log('Getting client')
        if not self._client:
            self._client = self.get_mgmt_svc_client(EventGridManagementClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)
        return self._client

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False
            ),
            odata_query=dict(
                type='str',
                required=False
            ),
            resource_group=dict(
                type='str',
                required=False
            ),
            tags=dict(
                type='list',
                elements='str'
            ),
        )

        mutually_exclusive = [('name', 'odata_query')]

        self._client = None

        self.name = None
        self.tags = None

        self.results = dict(
            changed=False,
            eventgrid_topics=[]
        )

        super(AzureRMEventgridTopic, self).__init__(self.module_arg_spec,
                                                    supports_tags=False,
                                                    supports_check_mode=True,
                                                    mutually_exclusive=mutually_exclusive,
                                                    facts_module=True,
                                                    )

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name is not None and self.resource_group is not None:
            self.results['eventgrid_topics'] = self.get_item(self.resource_group,
                                                             self.name)
        elif self.resource_group is not None:
            self.results['eventgrid_topics'] = self.list_by_resourcegroup(self.resource_group,
                                                                          self.odata_query)
        else:
            self.results['eventgrid_topics'] = self.list_items(self.odata_query)

        return self.results

    def list_by_resourcegroup(self, resource_group, odata_query=None):
        self.log('List all items by resource group')
        try:
            items = self.client.topics.list_by_resource_group(resource_group, odata_query)
        except ResourceNotFoundError as exc:
            self.fail("Failed to list all items in resource group - {0}".format(str(exc)))

        return self.filter_results(items)

    def list_items(self, odata_query=None):
        self.log('List all items by resource group')
        try:
            items = self.client.topics.list_by_subscription(odata_query)
        except ResourceNotFoundError as exc:
            self.fail("Failed to list all items in resource group - {0}".format(str(exc)))

        return self.filter_results(items)

    def get_item(self, resource_group, topic_name):
        '''
        Get the eventgrid topic

        :return: eventgrid topic dict
        '''
        try:
            response = self.client.topics.get(resource_group, topic_name)
            return self.filter_results([response])
        except ResourceNotFoundError:
            self.log('Did not find the eventgrid_topic.')
            return []
        except Exception as exc:
            self.fail('Error when get eventgrid_topic got Excetion as {0}'.format(str(exc)))

    def filter_results(self, items):
        results = []
        for item in items:
            topic = self.format_eventgrid_topic(item)
            if topic and self.has_tags(topic.get('tags', {}), self.tags):
                resource_group = self.parse_resource_to_dict(topic.get('id')).get('resource_group')
                topic_keys = self.client.topics.list_shared_access_keys(resource_group, topic.get('name'))
                topic['topic_keys'] = topic_keys and topic_keys.as_dict() or {}
                results.append(topic)
        return results

    def format_eventgrid_topic(self, topic_data):
        results = topic_data and topic_data.as_dict() or {}
        return results


def main():
    AzureRMEventgridTopic()


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The ios_ntp_global config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.ntp_global import (
    Ntp_globalTemplate,
)


class Ntp_global(ResourceModule):
    """
    The ios_ntp_global config class
    """

    def __init__(self, module):
        super(Ntp_global, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="ntp_global",
            tmplt=Ntp_globalTemplate(),
        )
        self.parsers = [
            "allow.control.rate_limit",
            "allow.private",
            "authenticate",
            "broadcast_delay",
            "clock_period",
            "logging",
            "master.enabled",
            "master.stratum",
            "max_associations",
            "max_distance",
            "min_distance",
            "orphan",
            "panic_update",
            "passive",
            "source",
            "update_calendar",
        ]
        self.complex_parser = [
            "peer",
            "query_only",
            "serve",
            "serve_only",
            "authentication_keys",
            "peers",
            "servers",
            "trusted_keys",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = self._ntp_list_to_dict(self.want)
        haved = self._ntp_list_to_dict(self.have)

        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        if self.state == "deleted":
            wantd = {}

        self._compare(want=wantd, have=haved)

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Ntp_global network resource.
        """
        self.compare(parsers=self.parsers, want=want, have=have)
        self._compare_access_groups(want, have)
        self._compare_lists_attrs(want, have)

    def _compare_lists_attrs(self, want, have):
        """Compare list of dict"""
        for _parser in self.complex_parser[4:8]:  # other list attrs
            i_want = want.get(_parser, {})
            i_have = have.get(_parser, {})
            for key, wanting in i_want.items():
                haveing = i_have.pop(key, {})
                if wanting != haveing:
                    if (
                        _parser != "authentication_keys"
                        and haveing
                        and self.state in ["overridden", "replaced"]
                    ):
                        self.addcmd(haveing, _parser, negate=True)
                    self.addcmd(wanting, _parser)
            for key, haveing in i_have.items():
                self.addcmd(haveing, _parser, negate=True)

    def _compare_access_groups(self, want, have):
        w = want.get("access_group", {})
        h = have.get("access_group", {})
        for _parser in self.complex_parser[0:4]:  # access_group
            i_want = w.get(_parser, {})
            i_have = h.get(_parser, {})

            for key, wanting in i_want.items():
                haveing = i_have.pop(key, {})
                if wanting != haveing:
                    self.addcmd(wanting, _parser)

            for key, haveing in i_have.items():
                self.addcmd(haveing, _parser, negate=True)

    def _ntp_list_to_dict(self, data):
        """Convert all list of dicts to dicts of dicts"""
        p_key = {
            "servers": "server",
            "peers": "peer",
            "authentication_keys": "id",
            "peer": "access_list",
            "query_only": "access_list",
            "serve": "access_list",
            "serve_only": "access_list",
            "trusted_keys": "range_start",
            "access_group": True,
        }
        tmp_data = deepcopy(data)
        for k, _v in p_key.items():
            if k in tmp_data and k != "access_group":
                if k in ["servers", "peers"]:
                    for i in tmp_data[k]:
                        if i.get("key"):
                            del i["key"]
                tmp_data[k] = {str(i[p_key[k]]): i for i in tmp_data[k]}
            elif tmp_data.get("access_group") and k == "access_group":
                tmp_data[k] = self._ntp_list_to_dict(tmp_data.get("access_group"))
        return tmp_data



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_cellular_sims
short_description: Resource module for devices _cellular _sims
description:
  - Manage operation update of the resource devices _cellular _sims.
  - Updates the SIM and APN configurations for a cellular device.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  serial:
    description: Serial path parameter.
    type: str
  simFailover:
    description: SIM Failover settings.
    suboptions:
      enabled:
        description: Failover to secondary SIM (optional).
        type: bool
      timeout:
        description: Failover timeout in seconds (optional).
        type: int
    type: dict
  simOrdering:
    description: Specifies the ordering of all SIMs for an MG primary, secondary,
      and not-in-use (when applicable). It's required for devices with 3 or more SIMs
      and can be used in place of 'isPrimary' for dual-SIM devices. To indicate eSIM,
      use 'sim3'. Sim failover will occur only between primary and secondary sim slots.
    elements: str
    type: list
  sims:
    description: List of SIMs. If a SIM was previously configured and not specified
      in this request, it will remain unchanged.
    elements: dict
    suboptions:
      apns:
        description: APN configurations. If empty, the default APN will be used.
        elements: dict
        suboptions:
          allowedIpTypes:
            description: IP versions to support (permitted values include 'ipv4',
              'ipv6').
            elements: str
            type: list
          authentication:
            description: APN authentication configurations.
            suboptions:
              password:
                description: APN password, if type is set (if APN password is not
                  supplied, the password is left unchanged).
                type: str
              type:
                description: APN auth type.
                type: str
              username:
                description: APN username, if type is set.
                type: str
            type: dict
          name:
            description: APN name.
            type: str
        type: list
      isPrimary:
        description: If true, this SIM is activated on platform bootup. It must be
          true on single-SIM devices and is a required field for dual-SIM MGs unless
          it is being configured using 'simOrdering'.
        type: bool
      simOrder:
        description: Priority of SIM slot being configured. Use a value between 1
          and total number of SIMs available. The value must be unique for each SIM.
        type: int
      slot:
        description: SIM slot being configured. Must be 'sim1' on single-sim devices.
          Use 'sim3' for eSIM.
        type: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for devices updateDeviceCellularSims
    description: Complete reference of the updateDeviceCellularSims API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-device-cellular-sims
notes:
  - SDK Method used are
    devices.Devices.update_device_cellular_sims,
  - Paths used are
    put /devices/{serial}/cellular/sims,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.devices_cellular_sims:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    serial: string
    simFailover:
      enabled: true
      timeout: 300
    simOrdering:
      - sim1
      - sim2
      - sim3
    sims:
      - apns:
          - allowedIpTypes:
              - ipv4
              - ipv6
            authentication:
              password: secret
              type: pap
              username: milesmeraki
            name: internet
        isPrimary: false
        simOrder: 3
        slot: sim1
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "sims": [
        {
          "slot": "string",
          "iccid": "string",
          "imsi": "string",
          "msisdn": "string",
          "isPrimary": true,
          "status": "string",
          "apns": [
            {
              "name": "string",
              "allowedIpTypes": [
                "string"
              ],
              "authentication": {
                "type": "string",
                "username": "string",
                "password": "string"
              }
            }
          ]
        }
      ],
      "simOrdering": [
        "string"
      ],
      "simFailover": {
        "enabled": true,
        "timeout": 0
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2023, Gaudenz Steinlin <gaudenz.steinlin@cloudscale.ch>
# Copyright: (c) 2023, Kenneth Joss <kenneth.joss@cloudscale.ch>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: load_balancer
short_description: Manages load balancers on the cloudscale.ch IaaS service
description:
  - Get, create, update, delete load balancers on the cloudscale.ch IaaS service.
notes:
  - If I(uuid) option is provided, it takes precedence over I(name) for load balancer selection. This allows to update the load balancers's name.
  - If no I(uuid) option is provided, I(name) is used for load balancer selection. If more than one load balancer with this name exists, execution is aborted.
author:
  - Gaudenz Steinlin (@gaudenz)
  - Kenneth Joss (@k-304)
version_added: "2.3.0"
options:
  state:
    description:
      - State of the load balancer.
    choices: [ present, absent ]
    default: present
    type: str
  name:
    description:
      - Name of the load balancer.
      - Either I(name) or I(uuid) are required.
    type: str
  uuid:
    description:
      - UUID of the load balancer.
      - Either I(name) or I(uuid) are required.
    type: str
  flavor:
    description:
      - Flavor of the load balancer.
    default: lb-standard
    type: str
  vip_addresses:
    description:
      - See the [API documentation](https://www.cloudscale.ch/en/api/v1#vip_addresses-attribute-specification) for details about this parameter.
    type: list
    elements: dict
    suboptions:
      subnet:
        description:
          - Create a VIP address on the subnet identified by this UUID.
        type: str
      address:
        description:
          - Use this address.
          - Must be in the same range as subnet.
          - If empty, a radom address will be used.
        type: str
  zone:
    description:
      - Zone in which the load balancer resides (e.g. C(lpg1) or C(rma1)).
    type: str
  tags:
    description:
      - Tags assosiated with the load balancer. Set this to C({}) to clear any tags.
    type: dict
extends_documentation_fragment: cloudscale_ch.cloud.api_parameters
'''

EXAMPLES = '''
# Create and start a load balancer
- name: Start cloudscale.ch load balancer
  cloudscale_ch.cloud.load_balancer:
    name: my-shiny-cloudscale-load-balancer
    flavor: lb-standard
    zone: rma1
    tags:
      project: my project
    api_token: xxxxxx

# Create and start a load balancer with specific subnet
- name: Start cloudscale.ch load balancer
  cloudscale_ch.cloud.load_balancer:
    name: my-shiny-cloudscale-load-balancer
    flavor: lb-standard
    vip_addresses:
      - subnet: d7b82c9b-5900-436c-9296-e94dca01c7a0
        address: 172.25.12.1
    zone: lpg1
    tags:
      project: my project
    api_token: xxxxxx

# Get load balancer facts by name
- name: Get facts of a load balancer
  cloudscale_ch.cloud.load_balancer:
    name: my-shiny-cloudscale-load-balancer
    api_token: xxxxxx
'''

RETURN = '''
href:
  description: API URL to get details about this load balancer
  returned: success when not state == absent
  type: str
  sample: https://api.cloudscale.ch/v1/load-balancers/0f62e0a7-f459-4fc4-9c25-9e57b6cb4b2f
uuid:
  description: The unique identifier for this load balancer
  returned: success
  type: str
  sample: cfde831a-4e87-4a75-960f-89b0148aa2cc
name:
  description: The display name of the load balancer
  returned: success
  type: str
  sample: web-lb
created_at:
  description: The creation date and time of the load balancer
  returned: success when not state == absent
  type: str
  sample: "2023-02-07T15:32:02.308041Z"
status:
  description: The current operational status of the load balancer
  returned: success
  type: str
  sample: running
state:
  description: The current state of the load balancer
  returned: success
  type: str
  sample: present
zone:
  description: The zone used for this load balancer
  returned: success when not state == absent
  type: dict
  sample: { 'slug': 'lpg1' }
flavor:
  description: The flavor that has been used for this load balancer
  returned: success when not state == absent
  type: list
  sample: { "slug": "lb-standard", "name": "LB-Standard" }
vip_addresses:
  description: List of vip_addresses for this load balancer
  returned: success when not state == absent
  type: dict
  sample: [ {"version": "4", "address": "192.0.2.110",
            "subnet": [
                "href": "https://api.cloudscale.ch/v1/subnets/92c70b2f-99cb-4811-8823-3d46572006e4",
                "uuid": "92c70b2f-99cb-4811-8823-3d46572006e4",
                "cidr": "192.0.2.0/24"
            ]} ]
tags:
  description: Tags assosiated with the load balancer
  returned: success
  type: dict
  sample: { 'project': 'my project' }
'''

from ansible.module_utils.basic import AnsibleModule
from ..module_utils.api import (
    AnsibleCloudscaleBase,
    cloudscale_argument_spec,
)

ALLOWED_STATES = ('present',
                  'absent',
                  )


class AnsibleCloudscaleLoadBalancer(AnsibleCloudscaleBase):

    def __init__(self, module):
        super(AnsibleCloudscaleLoadBalancer, self).__init__(
            module,
            resource_name='load-balancers',
            resource_create_param_keys=[
                'name',
                'flavor',
                'zone',
                'vip_addresses',
                'tags',
            ],
            resource_update_param_keys=[
                'name',
                'tags',
            ],
        )

    def create(self, resource, data=None):
        super().create(resource)
        if not self._module.check_mode:
            resource = self.wait_for_state('status', ('running', ))
        return resource


def main():
    argument_spec = cloudscale_argument_spec()
    argument_spec.update(dict(
        name=dict(type='str'),
        uuid=dict(type='str'),
        flavor=dict(type='str', default='lb-standard'),
        zone=dict(type='str'),
        vip_addresses=dict(
            type='list',
            elements='dict',
            options=dict(
                subnet=dict(type='str'),
                address=dict(type='str'),
            ),
        ),
        tags=dict(type='dict'),
        state=dict(type='str', default='present', choices=ALLOWED_STATES),
    ))

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=(),
        required_one_of=(('name', 'uuid'),),
        required_if=(('state', 'present', ('name',),),),
        supports_check_mode=True,
    )

    cloudscale_load_balancer = AnsibleCloudscaleLoadBalancer(module)
    cloudscale_load_balancer.query_constraint_keys = [
        'zone',
    ]

    if module.params['state'] == "absent":
        result = cloudscale_load_balancer.absent()
    else:
        result = cloudscale_load_balancer.present()
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# Python SDK Reference: https://learn.microsoft.com/en-us/python/api/azure-mgmt-cdn/azure.mgmt.cdn.operations.afdendpointsoperations?view=azure-python

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_afdendpoint_info
version_added: "2.7.0"
short_description: Get Azure Front Door Endpoint facts to be used with Standard or Premium Frontdoor Service
description:
    - Get facts for a specific Azure Front Door (AFD) Endpoint or all AFD Endpoints.
    - This differs from the Front Door classic service and only is intended to be used by the Standard or Premium service offering.

options:
    resource_group:
        description:
            - Name of the resource group where this AFD Profile belongs.
        required: true
        type: str
    profile_name:
        description:
            - Name of the AFD profile.
        required: true
        type: str
    name:
        description:
            - Limit results to a specific AFD Endpoint.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jarret Tooley (@jartoo)
'''

EXAMPLES = '''
- name: Get facts for all Endpoints in AFD Profile
  azure_rm_afdendpoint_info:
    resource_group: myResourceGroup
    profile_name: myCDNProfile
    tags:
      - key
      - key:value

- name: Get facts of specific AFD Endpoint
  azure_rm_afdendpoint_info:
    resource_group: myResourceGroup
    profile_name: myCDNProfile
    name: myEndpoint1
'''

RETURN = '''
afdendpoints:
    description: List of AFD Endpoints.
    returned: always
    type: complex
    contains:
        auto_generated_domain_name_label_scope:
            description:
                - Indicates the endpoint name reuse scope.
            type: str
            sample: TenantReuse
        deployment_status:
            description:
                - Current state of the resource.
            type: str
            sample: NotStarted
        enabled_state:
            description:
                - Whether to enable use of this rule.
            type: str
            sample: Enabled
        host_name:
            description:
                - The host name of the AFD Endpoint structured as endpointName.DNSZone.
            type: str
            sample: contoso.azureedge.net
        id:
            description:
                - ID of the AFD Endpoint.
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myCDN/providers/Microsoft.Cdn/profiles/myProf/endpoints/myEndpoint1"
        location:
            description:
                - Location of the AFD Endpoint.
            type: str
            sample: Global
        name:
            description:
                - Name of the AFD Endpoint.
            returned: always
            type: str
            sample: myEndpoint
        profile_name:
            description:
                - Name of the AFD Profile which holds the Endpoint.
            returned: always
            type: str
            sample: myProfile
        provisioning_state:
            description:
                - Provisioning status of the AFD Endpoint.
            type: str
            sample: Succeeded
        resource_group:
            description:
                - Name of a resource group where the AFD Endpoint exists.
            returned: always
            type: str
            sample: myResourceGroup
        tags:
            description:
                - The tags of the AFD Endpoint.
            type: list
            sample: foo
        type:
            description:
                - Resource type.
            type: str
            sample: Microsoft.Cdn/profiles/afdendpoints
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
import re

AZURE_OBJECT_CLASS = 'AFDEndpoint'


class AzureRMAFDEndpointInfo(AzureRMModuleBase):
    """Utility class to get Azure AFD Endpoint facts"""

    def __init__(self):

        self.module_args = dict(
            name=dict(type='str'),
            resource_group=dict(
                type='str',
                required=True
            ),
            profile_name=dict(
                type='str',
                required=True
            ),
            tags=dict(
                type='list',
                elements='str'
            )
        )

        self.results = dict(
            changed=False,
            afdendpoints=[]
        )

        self.name = None
        self.resource_group = None
        self.profile_name = None
        self.tags = None

        super(AzureRMAFDEndpointInfo, self).__init__(
            supports_check_mode=True,
            derived_arg_spec=self.module_args,
            supports_tags=False,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        if self.name:
            self.results['afdendpoints'] = self.get_item()
        else:
            self.results['afdendpoints'] = self.list_by_profile()

        return self.results

    def get_item(self):
        """Get a single Azure AFD Endpoint"""

        self.log('Get properties for {0}'.format(self.name))

        item = None
        result = []

        try:
            item = self.cdn_client.afd_endpoints.get(self.resource_group, self.profile_name, self.name)
        except Exception:
            pass

        if item and self.has_tags(item.tags, self.tags):
            result = [self.serialize_afdendpoint(item)]

        return result

    def list_by_profile(self):
        """Get all Azure AFD Endpoints within an AFD profile"""

        self.log('List all AFD Endpoints within an AFD profile')

        try:
            response = self.cdn_client.afd_endpoints.list_by_profile(self.resource_group, self.profile_name)
        except Exception as exc:
            self.fail('Failed to list all items - {0}'.format(str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                results.append(self.serialize_afdendpoint(item))

        return results

    def serialize_afdendpoint(self, afdendpoint):
        '''
        Convert a AFD Endpoint object to dict.
        :param afdendpoint: AFD Endpoint object
        :return: dict
        '''
        result = self.serialize_obj(afdendpoint, AZURE_OBJECT_CLASS)

        new_result = {}
        new_result['auto_generated_domain_name_label_scope'] = afdendpoint.auto_generated_domain_name_label_scope
        new_result['deployment_status'] = afdendpoint.deployment_status
        new_result['enabled_state'] = afdendpoint.enabled_state
        new_result['host_name'] = afdendpoint.host_name
        new_result['id'] = afdendpoint.id
        new_result['location'] = afdendpoint.location
        new_result['name'] = afdendpoint.name
        new_result['profile_name'] = re.sub('\\/.*', '', re.sub('.*profiles\\/', '', result['id']))
        new_result['provisioning_state'] = afdendpoint.provisioning_state
        new_result['resource_group'] = re.sub('\\/.*', '', re.sub('.*resourcegroups\\/', '', result['id']))
        new_result['tags'] = afdendpoint.tags
        new_result['type'] = afdendpoint.type
        return new_result


def main():
    """Main module execution code path"""

    AzureRMAFDEndpointInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_san_connectivity_policy
short_description: Manage SAN Connectivity Policies and vHBAs for Cisco Intersight
description:
  - Create, update, and delete SAN Connectivity Policies on Cisco Intersight.
  - Manage individual vHBAs (virtual Host Bus Adapters) within SAN Connectivity policies.
  - Supports Standalone and FIAttached target platforms with different configuration options.
  - Note SAN Connectivity policies are not supported on Unified Edge Servers.
  - SAN Connectivity policies define storage connectivity settings for server profiles.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/vnic/SanConnectivityPolicy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
      - Use 'default' for the default organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the SAN Connectivity Policy.
      - Must be unique within the organization.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the SAN Connectivity Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  target_platform:
    description:
      - The platform type for which the SAN Connectivity policy is intended.
      - C(Standalone) for Standalone servers.
      - C(FIAttached) for fabric interconnect attached servers.
      - Note SAN Connectivity policies are not supported on unified edge servers.
    type: str
    choices: ['Standalone', 'FIAttached']
    default: 'Standalone'
  placement_mode:
    description:
      - The mode used for placement of vHBAs on network adapters.
      - C(custom) requires manual placement specification for each vHBA.
      - C(auto) automatically distributes vHBAs between adapters during profile deployment.
      - Only applicable for FIAttached target platform.
    type: str
    choices: ['custom', 'auto']
    default: 'custom'
  wwnn_address_type:
    description:
      - Type of WWNN address assignment.
      - C(pool) to use a WWNN pool.
      - C(static) to manually assign a static WWNN address.
      - Only applicable when target_platform is FIAttached.
    type: str
    choices: ['pool', 'static']
    default: 'pool'
  wwnn_pool:
    description:
      - The WWNN pool that is assigned for WWNN address assignment.
      - Required when wwnn_address_type is pool and target platform is FIAttached.
      - Only applicable for FIAttached target platform.
    type: str
  static_wwnn_address:
    description:
      - The WWNN address for the server node must be in hexadecimal format xx:xx:xx:xx:xx:xx:xx:xx.
      - Allowed ranges are 20:00:00:00:00:00:00:00 to 20:FF:FF:FF:FF:FF:FF:FF or from 50:00:00:00:00:00:00:00 to 5F:FF:FF:FF:FF:FF:FF:FF.
      - To ensure uniqueness of WWN's in the SAN fabric, you are strongly encouraged to use the WWN prefix - 20:00:00:25:B5:xx:xx:xx.
      - Required when wwnn_address_type is static and target platform is FIAttached.
      - Only applicable for FIAttached target platform.
    type: str
  vhbas:
    description:
      - List of vHBAs to be created and attached to the SAN Connectivity policy.
      - Each vHBA will be configured with the specified fibre channel policies.
      - Configuration options vary based on target_platform.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Name of the virtual fibre channel interface.
          - Must be unique within the SAN Connectivity policy.
        type: str
        required: true
      state:
        description:
          - Whether to create/update or delete the vHBA.
        type: str
        choices: ['present', 'absent']
        default: present
      vhba_type:
        description:
          - vHBA Type configuration for SAN Connectivity Policy.
          - This configuration is supported only on Cisco VIC 14XX series and higher series of adapters.
          - Required when vHBA state is present.
        type: str
        choices: ['fc-initiator', 'fc-nvme-initiator', 'fc-nvme-target', 'fc-target']
      slot_id:
        description:
          - PCIe Slot where the VIC adapter is installed.
          - Supported values are (1-15) and MLOM.
          - Required for Standalone when vHBA state is present.
          - Required for FIAttached when vHBA state is present and auto_slot_id is false.
        type: str
      pci_link:
        description:
          - The PCI Link used as transport for the virtual interface.
          - PCI Link is only applicable for select Cisco UCS VIC 1300 models (UCSC-PCIE-C40Q-03, UCSB-MLOM-40G-03, UCSB-VIC-M83-8P) that support two PCI links.
          - The value, if specified, for any other VIC model will be ignored.
          - For Standalone, this is used directly.
          - For FIAttached, required when pci_link_assignment_mode is Custom.
        type: int
        choices: [0, 1]
        default: 0
      uplink_port:
        description:
          - Adapter port on which the virtual interface will be created.
          - Only applicable for Standalone platform.
        type: int
        choices: [0, 1, 2, 3]
        default: 0
      pci_order:
        description:
          - The order in which the virtual interface is brought up.
          - The order assigned to an interface should be unique for all the Ethernet and Fibre-Channel interfaces on each PCI link on a VIC adapter.
          - The order should start from zero with no overlaps.
          - The maximum value of PCI order is limited by the number of virtual interfaces (Ethernet and Fibre-Channel) on each PCI link on a VIC adapter.
          - All VIC adapters have a single PCI link except VIC 1340, VIC 1380 and VIC 1385 which have two.
        type: int
        default: 0
      persistent_lun_bindings:
        description:
          - Enables retention of LUN ID associations in memory until they are manually cleared.
        type: bool
        default: false
      fibre_channel_network_policy:
        description:
          - Name of the Fibre Channel Network Policy.
          - Required when vHBA state is present.
        type: str
      fibre_channel_qos_policy:
        description:
          - Name of the Fibre Channel QoS Policy.
          - Required when vHBA state is present.
        type: str
      fibre_channel_adapter_policy:
        description:
          - Name of the Fibre Channel Adapter Policy.
          - Required when vHBA state is present.
        type: str
      pin_group_name:
        description:
          - Pingroup name associated to vHBA for static pinning.
          - SCP deploy will resolve pingroup name and fetches the corresponding uplink port/port channel to pin the vHBA traffic.
          - Optional field for FIAttached target platform.
        type: str
      wwpn_address_type:
        description:
          - Type of WWPN address assignment.
          - C(pool) to use a WWPN pool.
          - C(static) to manually assign a static WWPN address.
          - Only applicable for FIAttached target platform.
        type: str
        choices: ['pool', 'static']
        default: 'pool'
      wwpn_pool:
        description:
          - The WWPN pool used for assigning the WWPN address to the vHBA.
          - Required when wwpn_address_type is pool and target platform is FIAttached.
          - Only applicable for FIAttached target platform.
        type: str
      static_wwpn_address:
        description:
          - The WWPN address must be in hexadecimal format xx:xx:xx:xx:xx:xx:xx:xx.
          - Allowed ranges are 20:00:00:00:00:00:00:00 to 20:FF:FF:FF:FF:FF:FF:FF or from 50:00:00:00:00:00:00:00 to 5F:FF:FF:FF:FF:FF:FF:FF.
          - To ensure uniqueness of WWN's in the SAN fabric, you are strongly encouraged to use the WWN prefix - 20:00:00:25:B5:xx:xx:xx.
          - Required when wwpn_address_type is static and target platform is FIAttached.
          - Only applicable for FIAttached target platform.
        type: str
      switch_id:
        description:
          - The fabric port to which the vHBA will be associated.
          - Only applicable for FIAttached target platform.
        type: str
        choices: ['a', 'b']
        default: 'a'
      auto_slot_id:
        description:
          - Enable or disable automatic assignment of the VIC slot ID.
          - If enabled and the server has only one VIC, the same VIC is chosen for all the vHBAs.
          - If enabled and the server has multiple VICs, the vHBA is deployed on the first VIC.
          - If disabled, slot_id must be specified.
          - Only applicable for FIAttached target platform.
        type: bool
        default: true
      auto_pci_link:
        description:
          - Enable or disable automatic assignment of the PCI Link in a dual-link adapter.
          - This option applies only to 13xx series VICs that support dual-link.
          - If enabled, the system determines the placement of the vHBA on either of the PCI Links.
          - If disabled, pci_link_assignment_mode must be specified.
          - Only applicable for FIAttached target platform.
        type: bool
        default: true
      pci_link_assignment_mode:
        description:
          - PCI Link assignment mode when auto_pci_link is disabled.
          - C(custom) allows manual selection of PCI link via pci_link parameter.
          - C(load-balanced) automatically distributes vHBAs across available PCI links.
          - Required when auto_pci_link is false.
          - Only applicable for FIAttached target platform.
        type: str
        choices: ['custom', 'load-balanced']
      fibre_channel_zone_policies:
        description:
          - List of Fibre Channel Zone Policy names.
          - Relationship to the FC Zone policy to configure Zones on the switch.
          - Optional field for FIAttached target platform.
        type: list
        elements: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a SAN Connectivity Policy for Standalone servers
  cisco.intersight.intersight_san_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "standalone-san-policy"
    description: "SAN connectivity policy for standalone servers"
    target_platform: "Standalone"
    tags:
      - Key: "Environment"
        Value: "Production"
      - Key: "Site"
        Value: "DataCenter-A"
    vhbas:
      - name: "vhba0"
        vhba_type: "fc-initiator"
        slot_id: "1"
        pci_link: 0
        uplink_port: 0
        pci_order: 0
        persistent_lun_bindings: false
        fibre_channel_network_policy: "fc-network-policy"
        fibre_channel_qos_policy: "fc-qos-policy"
        fibre_channel_adapter_policy: "fc-adapter-policy"
      - name: "vhba1"
        vhba_type: "fc-nvme-initiator"
        slot_id: "4"
        pci_link: 0
        uplink_port: 2
        pci_order: 5
        persistent_lun_bindings: true
        fibre_channel_network_policy: "fc-network-policy"
        fibre_channel_qos_policy: "fc-qos-policy"
        fibre_channel_adapter_policy: "fc-adapter-policy"
    state: present

- name: Create a SAN Connectivity Policy with auto placement
  cisco.intersight.intersight_san_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "auto-placement-san-policy"
    description: "SAN policy with automatic vHBA placement"
    target_platform: "FIAttached"
    placement_mode: "auto"
    wwnn_address_type: "pool"
    wwnn_pool: "default-wwnn-pool"
    vhbas:
      - name: "vhba-auto-1"
        vhba_type: "fc-initiator"
        wwpn_address_type: "pool"
        wwpn_pool: "default-wwpn-pool"
        switch_id: "a"
        fibre_channel_network_policy: "fc-network-policy"
        fibre_channel_qos_policy: "fc-qos-policy"
        fibre_channel_adapter_policy: "fc-adapter-policy"
      - name: "vhba-auto-2"
        vhba_type: "fc-initiator"
        wwpn_address_type: "pool"
        wwpn_pool: "default-wwpn-pool"
        switch_id: "b"
        fibre_channel_network_policy: "fc-network-policy"
        fibre_channel_qos_policy: "fc-qos-policy"
        fibre_channel_adapter_policy: "fc-adapter-policy"
    state: present

- name: Create a SAN Connectivity Policy for FI-Attached servers with WWNN pool
  cisco.intersight.intersight_san_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fi-attached-san-policy"
    description: "SAN connectivity policy for FI-attached servers"
    target_platform: "FIAttached"
    placement_mode: "custom"
    wwnn_address_type: "pool"
    wwnn_pool: "default-wwnn-pool"
    tags:
      - Key: "Environment"
        Value: "Production"
    vhbas:
      - name: "vhba-a"
        vhba_type: "fc-initiator"
        wwpn_address_type: "pool"
        wwpn_pool: "default-wwpn-pool"
        switch_id: "a"
        auto_slot_id: true
        auto_pci_link: true
        pci_order: 0
        persistent_lun_bindings: false
        fibre_channel_network_policy: "fc-network-policy"
        fibre_channel_qos_policy: "fc-qos-policy"
        fibre_channel_adapter_policy: "fc-adapter-policy"
        fibre_channel_zone_policies:
          - "fc-zone-policy-1"
      - name: "vhba-b"
        vhba_type: "fc-initiator"
        wwpn_address_type: "pool"
        wwpn_pool: "default-wwpn-pool"
        switch_id: "b"
        auto_slot_id: true
        auto_pci_link: true
        pci_order: 1
        persistent_lun_bindings: false
        fibre_channel_network_policy: "fc-network-policy"
        fibre_channel_qos_policy: "fc-qos-policy"
        fibre_channel_adapter_policy: "fc-adapter-policy"
        fibre_channel_zone_policies:
          - "fc-zone-policy-1"
    state: present

- name: Create a SAN Connectivity Policy for FI-Attached with static WWPN
  cisco.intersight.intersight_san_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fi-attached-static-san-policy"
    description: "SAN connectivity policy with static addresses"
    target_platform: "FIAttached"
    placement_mode: "custom"
    wwnn_address_type: "pool"
    wwnn_pool: "default-wwnn-pool"
    vhbas:
      - name: "vhba-static"
        vhba_type: "fc-target"
        wwpn_address_type: "static"
        static_wwpn_address: "50:00:00:00:00:00:00:00"
        switch_id: "b"
        auto_slot_id: false
        slot_id: "2"
        auto_pci_link: false
        pci_link_assignment_mode: "load-balanced"
        pci_order: 1
        persistent_lun_bindings: true
        fibre_channel_network_policy: "fc-network-policy"
        fibre_channel_qos_policy: "fc-qos-policy"
        fibre_channel_adapter_policy: "fc-adapter-policy"
        fibre_channel_zone_policies:
          - "fc-zone-policy-1"
    state: present

- name: Create a SAN Connectivity Policy for FI-Attached with advanced placement
  cisco.intersight.intersight_san_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fi-attached-advanced-placement"
    description: "SAN connectivity with custom placement"
    target_platform: "FIAttached"
    placement_mode: "custom"
    wwnn_address_type: "static"
    static_wwnn_address: "20:00:00:25:B5:00:00:01"
    vhbas:
      - name: "vhba-custom"
        vhba_type: "fc-initiator"
        wwpn_address_type: "pool"
        wwpn_pool: "wwpn-pool"
        switch_id: "a"
        auto_slot_id: false
        slot_id: "MLOM"
        auto_pci_link: false
        pci_link_assignment_mode: "custom"
        pci_link: 1
        pci_order: 3
        persistent_lun_bindings: false
        pin_group_name: "pingroup-a"
        fibre_channel_network_policy: "fc-network-policy"
        fibre_channel_qos_policy: "fc-qos-policy"
        fibre_channel_adapter_policy: "fc-adapter-policy"
    state: present

- name: Update SAN connectivity policy - manage vHBA states
  cisco.intersight.intersight_san_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "mixed-vhba-states-policy"
    description: "Policy demonstrating vHBA state management"
    target_platform: "Standalone"
    vhbas:
      - name: "vhba0"
        vhba_type: "fc-initiator"
        slot_id: "1"
        pci_link: 0
        uplink_port: 0
        pci_order: 0
        persistent_lun_bindings: false
        fibre_channel_network_policy: "fc-network-policy"
        fibre_channel_qos_policy: "fc-qos-policy"
        fibre_channel_adapter_policy: "fc-adapter-policy"
        state: present
      - name: "vhba-old"
        state: absent
      - name: "vhba-new"
        vhba_type: "fc-nvme-initiator"
        slot_id: "3"
        pci_link: 0
        uplink_port: 1
        pci_order: 1
        persistent_lun_bindings: true
        fibre_channel_network_policy: "fc-network-policy"
        fibre_channel_qos_policy: "fc-qos-policy"
        fibre_channel_adapter_policy: "fc-adapter-policy"
        state: present
    state: present

- name: Delete a SAN Connectivity Policy
  cisco.intersight.intersight_san_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "old-san-policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "fi-attached-san-policy",
        "ObjectType": "vnic.SanConnectivityPolicy",
        "TargetPlatform": "FIAttached",
        "WwnnAddressType": "POOL",
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ],
        "vHBAs": [
            {
                "Name": "vhba-a",
                "ObjectType": "vnic.FcIf",
                "Type": "fc-initiator",
                "Order": 0,
                "Placement": {
                    "SwitchId": "A",
                    "AutoSlotId": true,
                    "AutoPciLink": true
                }
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec
from ansible_collections.cisco.intersight.plugins.module_utils.intersight_vhba_utils import (
    validate_wwnn_address_config, validate_wwpn_address_config, validate_fi_attached_placement_config,
    validate_standalone_vhba_config, resolve_policy_moids_from_mappings,
    resolve_fc_zone_policies, build_wwpn_address_config, build_fi_attached_placement,
    build_standalone_placement, get_san_connectivity_vhba_policy_mappings
)


def validate_fi_attached_params(module):
    """
    Validate FI-Attached specific parameters.
    """
    validate_wwnn_address_config(module, module.params)


def validate_fi_attached_vhba_config(module, vhba_config):
    """
    Validate FIAttached specific vHBA configuration.
    """
    vhba_name = vhba_config.get('name', 'unknown')
    validate_wwpn_address_config(module, vhba_config, f"vHBA '{vhba_name}'")
    validate_fi_attached_placement_config(module, vhba_config)


def validate_input(module):
    """
    Validate module input parameters.
    """
    target_platform = module.params.get('target_platform')
    if target_platform == 'FIAttached':
        validate_fi_attached_params(module)
    vhbas = module.params.get('vhbas', [])
    for vhba_config in vhbas:
        vhba_name = vhba_config.get('name')
        vhba_state = vhba_config.get('state', 'present')
        if vhba_state == 'present':
            required_fields = ['vhba_type', 'fibre_channel_network_policy', 'fibre_channel_qos_policy', 'fibre_channel_adapter_policy']
            for field in required_fields:
                if not vhba_config.get(field):
                    module.fail_json(msg=f"{field} is required when vHBA state is 'present' for vHBA '{vhba_name}'")
            if target_platform == 'FIAttached':
                validate_fi_attached_vhba_config(module, vhba_config)
            elif target_platform == 'Standalone':
                validate_standalone_vhba_config(module, vhba_config)


def build_standalone_vhba_api_body(intersight, policy_cache, module, vhba_config, san_connectivity_policy_moid):
    """
    Build Standalone vHBA API body for Intersight API call.
    """
    organization_name = module.params['organization']
    target_platform = module.params.get('target_platform')
    vhba_api_body = {
        'Name': vhba_config['name'],
        'Type': vhba_config['vhba_type'],
        'Placement': build_standalone_placement(vhba_config),
        'Order': vhba_config.get('pci_order', 0),
        'PersistentBindings': vhba_config.get('persistent_lun_bindings', False),
        'SanConnectivityPolicy': san_connectivity_policy_moid,
        'StaticWwpnAddress': ''
    }
    # Resolve FC policy MOIDs
    policy_mappings = get_san_connectivity_vhba_policy_mappings(target_platform)
    policy_moids = resolve_policy_moids_from_mappings(intersight, policy_cache, module, vhba_config, policy_mappings, organization_name)
    vhba_api_body.update(policy_moids)
    vhba_api_body['FcZonePolicies'] = []
    return vhba_api_body


def build_fi_attached_vhba_api_body(intersight, policy_cache, module, vhba_config, san_connectivity_policy_moid):
    """
    Build FIAttached vHBA API body for Intersight API call.
    """
    organization_name = module.params['organization']
    target_platform = module.params.get('target_platform')
    vhba_api_body = {
        'Name': vhba_config['name'],
        'Type': vhba_config['vhba_type'],
        'Order': vhba_config.get('pci_order', 0),
        'PersistentBindings': vhba_config.get('persistent_lun_bindings', False),
        'SanConnectivityPolicy': san_connectivity_policy_moid
    }
    # Build WWPN address configuration
    wwpn_config = build_wwpn_address_config(vhba_config, intersight, policy_cache, module, organization_name)
    vhba_api_body.update(wwpn_config)

    # Build placement configuration
    vhba_api_body['Placement'] = build_fi_attached_placement(vhba_config)

    # Resolve FC policy MOIDs
    policy_mappings = get_san_connectivity_vhba_policy_mappings(target_platform)
    policy_moids = resolve_policy_moids_from_mappings(intersight, policy_cache, module, vhba_config, policy_mappings, organization_name)
    vhba_api_body.update(policy_moids)

    # Resolve FC Zone Policies (optional)
    fc_zone_policy_names = vhba_config.get('fibre_channel_zone_policies')
    fc_zone_policy_moids = resolve_fc_zone_policies(intersight, policy_cache, module, fc_zone_policy_names, organization_name)
    vhba_api_body['FcZonePolicies'] = fc_zone_policy_moids

    # Add pin group name if specified
    if vhba_config.get('pin_group_name'):
        vhba_api_body['PinGroupName'] = vhba_config['pin_group_name']

    return vhba_api_body


def build_vhba_api_body(intersight, policy_cache, module, vhba_config, san_connectivity_policy_moid):
    """
    Build vHBA API body based on target platform.
    """
    target_platform = module.params.get('target_platform')
    if target_platform == 'FIAttached':
        return build_fi_attached_vhba_api_body(intersight, policy_cache, module, vhba_config, san_connectivity_policy_moid)
    else:
        return build_standalone_vhba_api_body(intersight, policy_cache, module, vhba_config, san_connectivity_policy_moid)


def main():
    vhba_options = dict(
        name=dict(type='str', required=True),
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        vhba_type=dict(type='str', choices=['fc-initiator', 'fc-nvme-initiator', 'fc-nvme-target', 'fc-target']),
        slot_id=dict(type='str'),
        pci_link=dict(type='int', choices=[0, 1], default=0),
        uplink_port=dict(type='int', choices=[0, 1, 2, 3], default=0),
        pci_order=dict(type='int', default=0),
        persistent_lun_bindings=dict(type='bool', default=False),
        fibre_channel_network_policy=dict(type='str'),
        fibre_channel_qos_policy=dict(type='str'),
        fibre_channel_adapter_policy=dict(type='str'),
        pin_group_name=dict(type='str'),
        wwpn_address_type=dict(type='str', choices=['pool', 'static'], default='pool'),
        wwpn_pool=dict(type='str'),
        static_wwpn_address=dict(type='str'),
        switch_id=dict(type='str', choices=['a', 'b'], default='a'),
        auto_slot_id=dict(type='bool', default=True),
        auto_pci_link=dict(type='bool', default=True),
        pci_link_assignment_mode=dict(type='str', choices=['custom', 'load-balanced']),
        fibre_channel_zone_policies=dict(type='list', elements='str')
    )
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        target_platform=dict(type='str', choices=['Standalone', 'FIAttached'], default='Standalone'),
        placement_mode=dict(type='str', choices=['custom', 'auto'], default='custom'),
        wwnn_address_type=dict(type='str', choices=['pool', 'static'], default='pool'),
        wwnn_pool=dict(type='str'),
        static_wwnn_address=dict(type='str'),
        vhbas=dict(type='list', elements='dict', options=vhba_options)
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    if module.params['state'] == 'present':
        validate_input(module)

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/vnic/SanConnectivityPolicies'
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()
        target_platform_map = {
            'Standalone': 'Standalone',
            'FIAttached': 'FIAttached'
        }
        api_target_platform = target_platform_map[intersight.module.params['target_platform']]
        intersight.api_body['TargetPlatform'] = api_target_platform

        if intersight.module.params['target_platform'] == 'FIAttached':
            # PlacementMode is only applicable for FIAttached
            placement_mode_map = {'custom': 'custom', 'auto': 'auto'}
            api_placement_mode = placement_mode_map[intersight.module.params['placement_mode']]
            intersight.api_body['PlacementMode'] = api_placement_mode
            wwnn_address_type_map = {'pool': 'POOL', 'static': 'STATIC'}
            api_wwnn_address_type = wwnn_address_type_map[intersight.module.params['wwnn_address_type']]
            intersight.api_body['WwnnAddressType'] = api_wwnn_address_type

            if intersight.module.params['wwnn_address_type'] == 'pool':
                wwnn_pool_moid = intersight.get_moid_by_name_and_org(
                    resource_path='/fcpool/Pools',
                    resource_name=intersight.module.params['wwnn_pool'],
                    organization_name=intersight.module.params['organization']
                )
                if not wwnn_pool_moid:
                    intersight.module.fail_json(
                        msg=f"WWNN Pool '{intersight.module.params['wwnn_pool']}' not found in organization '{intersight.module.params['organization']}'"
                    )
                intersight.api_body['WwnnPool'] = wwnn_pool_moid
                intersight.api_body['StaticWwnnAddress'] = ''
            else:
                intersight.api_body['StaticWwnnAddress'] = intersight.module.params['static_wwnn_address']

        elif intersight.module.params['target_platform'] == 'Standalone':
            intersight.api_body['PlacementMode'] = 'custom'
            intersight.api_body['WwnnAddressType'] = 'POOL'
            intersight.api_body['StaticWwnnAddress'] = ''

    intersight.configure_policy_or_profile(resource_path=resource_path)
    san_connectivity_policy_response = intersight.result['api_response']
    san_connectivity_policy_moid = None
    if intersight.module.params['state'] == 'present' and san_connectivity_policy_response:
        san_connectivity_policy_moid = san_connectivity_policy_response.get('Moid')

    # Process vHBAs
    vhbas_response = []
    if intersight.module.params['state'] == 'present' and intersight.module.params.get('vhbas'):
        policy_cache = {}
        for vhba_config in intersight.module.params['vhbas']:
            vhba_state = vhba_config.get('state', 'present')
            if vhba_state == 'present':
                vhba_api_body = build_vhba_api_body(
                    intersight, policy_cache, module, vhba_config, san_connectivity_policy_moid
                )
                intersight.api_body = vhba_api_body

            resource_path = '/vnic/FcIfs'
            custom_filter = f"Name eq '{vhba_config['name']}' and SanConnectivityPolicy.Moid eq '{san_connectivity_policy_moid}'"
            intersight.configure_secondary_resource(
                resource_path=resource_path,
                state=vhba_state,
                custom_filter=custom_filter
            )
            if vhba_state == 'present':
                vhbas_response.append(intersight.result['api_response'])

    if san_connectivity_policy_response:
        san_connectivity_policy_response['vHBAs'] = vhbas_response
        intersight.result['api_response'] = san_connectivity_policy_response

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c) 2017-present Alibaba Group Holding Limited. He Guimin <heguimin36@163.com>
#
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import json
import os
import traceback
import typing as t

from ansible.module_utils.basic import env_fallback

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule

try:
    import footmark
    import footmark.dns
    import footmark.ecs
    import footmark.ess
    import footmark.market
    import footmark.ram
    import footmark.rds
    import footmark.slb
    import footmark.sts
    import footmark.vpc

    FOOTMARK_IMP_ERR = None
    HAS_FOOTMARK = True
except ImportError:
    FOOTMARK_IMP_ERR = traceback.format_exc()
    HAS_FOOTMARK = False


class AnsibleACSError(Exception):
    pass


def acs_common_argument_spec():
    return dict(
        alicloud_access_key=dict(
            aliases=["access_key_id", "access_key"],
            no_log=True,
            fallback=(env_fallback, ["ALICLOUD_ACCESS_KEY", "ALICLOUD_ACCESS_KEY_ID"]),
        ),
        alicloud_secret_key=dict(
            aliases=["secret_access_key", "secret_key"],
            no_log=True,
            fallback=(env_fallback, ["ALICLOUD_SECRET_KEY", "ALICLOUD_SECRET_ACCESS_KEY"]),
        ),
        alicloud_security_token=dict(
            aliases=["security_token"], no_log=True, fallback=(env_fallback, ["ALICLOUD_SECURITY_TOKEN"])
        ),
        ecs_role_name=dict(aliases=["role_name"], fallback=(env_fallback, ["ALICLOUD_ECS_ROLE_NAME"])),
    )


def ecs_argument_spec():
    spec = acs_common_argument_spec()
    spec.update(
        dict(
            alicloud_region=dict(
                required=True,
                aliases=["region", "region_id"],
                fallback=(env_fallback, ["ALICLOUD_REGION", "ALICLOUD_REGION_ID"]),
            ),
            alicloud_assume_role_arn=dict(
                fallback=(env_fallback, ["ALICLOUD_ASSUME_ROLE_ARN"]), aliases=["assume_role_arn"]
            ),
            alicloud_assume_role_session_name=dict(
                fallback=(env_fallback, ["ALICLOUD_ASSUME_ROLE_SESSION_NAME"]), aliases=["assume_role_session_name"]
            ),
            alicloud_assume_role_session_expiration=dict(
                type="int",
                fallback=(env_fallback, ["ALICLOUD_ASSUME_ROLE_SESSION_EXPIRATION"]),
                aliases=["assume_role_session_expiration"],
            ),
            alicloud_assume_role=dict(type="dict", aliases=["assume_role"]),
            profile=dict(fallback=(env_fallback, ["ALICLOUD_PROFILE"])),
            shared_credentials_file=dict(fallback=(env_fallback, ["ALICLOUD_SHARED_CREDENTIALS_FILE"])),
        )
    )
    return spec


def get_acs_connection_info(params):
    ecs_params = dict(
        acs_access_key_id=params.get("alicloud_access_key"),
        acs_secret_access_key=params.get("alicloud_secret_key"),
        security_token=params.get("alicloud_security_token"),
        ecs_role_name=params.get("ecs_role_name"),
        user_agent="Ansible-Provider-Alicloud",
    )
    return ecs_params


def connect_to_acs(acs_module, region, **params):
    conn = acs_module.connect_to_region(region, **params)
    if not conn:
        if region not in [acs_module_region.id for acs_module_region in acs_module.regions()]:
            raise AnsibleACSError(
                f"Region {region} does not seem to be available for acs module {acs_module.__name__}."
            )
        else:
            raise AnsibleACSError(
                f"Unknown problem connecting to region {region} for acs module {acs_module.__name__}."
            )
    return conn


def get_assume_role(params):
    """Return new params"""
    sts_params = get_acs_connection_info(params)
    assume_role = {}
    if params.get("assume_role"):
        assume_role["alicloud_assume_role_arn"] = params["assume_role"].get("role_arn")
        assume_role["alicloud_assume_role_session_name"] = params["assume_role"].get("session_name")
        assume_role["alicloud_assume_role_session_expiration"] = params["assume_role"].get("session_expiration")
        assume_role["alicloud_assume_role_policy"] = params["assume_role"].get("policy")

    assume_role_params = {
        "role_arn": params.get("alicloud_assume_role_arn")
        if params.get("alicloud_assume_role_arn")
        else assume_role.get("alicloud_assume_role_arn"),
        "role_session_name": params.get("alicloud_assume_role_session_name")
        if params.get("alicloud_assume_role_session_name")
        else assume_role.get("alicloud_assume_role_session_name"),
        "duration_seconds": params.get("alicloud_assume_role_session_expiration")
        if params.get("alicloud_assume_role_session_expiration")
        else assume_role.get("alicloud_assume_role_session_expiration", 3600),
        "policy": assume_role.get("alicloud_assume_role_policy", {}),
    }

    try:
        sts = (
            connect_to_acs(footmark.sts, params.get("alicloud_region"), **sts_params)
            .assume_role(**assume_role_params)
            .read()
        )
        sts_params["acs_access_key_id"], sts_params["acs_secret_access_key"], sts_params["security_token"] = (
            sts["access_key_id"],
            sts["access_key_secret"],
            sts["security_token"],
        )
    except AnsibleACSError as e:
        params.fail_json(msg=str(e))
    return sts_params


def get_profile(params):
    if not params["alicloud_access_key"] and not params["ecs_role_name"] and params["profile"]:
        path = (
            params["shared_credentials_file"]
            if params["shared_credentials_file"]
            else f"{os.getenv('HOME')}/.aliyun/config.json"
        )
        auth = {}
        with open(path) as f:
            for pro in json.load(f)["profiles"]:
                if params["profile"] == pro["name"]:
                    auth = pro
        if auth:
            if auth["mode"] == "AK" and auth.get("access_key_id") and auth.get("access_key_secret"):
                params["alicloud_access_key"] = auth.get("access_key_id")
                params["alicloud_secret_key"] = auth.get("access_key_secret")
                params["alicloud_region"] = auth.get("region_id")
                params = get_acs_connection_info(params)
            elif (
                auth["mode"] == "StsToken"
                and auth.get("access_key_id")
                and auth.get("access_key_secret")
                and auth.get("sts_token")
            ):
                params["alicloud_access_key"] = auth.get("access_key_id")
                params["alicloud_secret_key"] = auth.get("access_key_secret")
                params["security_token"] = auth.get("sts_token")
                params["alicloud_region"] = auth.get("region_id")
                params = get_acs_connection_info(params)
            elif auth["mode"] == "EcsRamRole":
                params["ecs_role_name"] = auth.get("ram_role_name")
                params["alicloud_region"] = auth.get("region_id")
                params = get_acs_connection_info(params)
            elif auth["mode"] == "RamRoleArn" and auth.get("ram_role_arn"):
                params["alicloud_access_key"] = auth.get("access_key_id")
                params["alicloud_secret_key"] = auth.get("access_key_secret")
                params["security_token"] = auth.get("sts_token")
                params["ecs_role_name"] = auth.get("ram_role_name")
                params["alicloud_assume_role_arn"] = auth.get("ram_role_arn")
                params["alicloud_assume_role_session_name"] = auth.get("ram_session_name")
                params["alicloud_assume_role_session_expiration"] = auth.get("expired_seconds")
                params["alicloud_region"] = auth.get("region_id")
                params = get_assume_role(params)
    elif params.get("alicloud_assume_role_arn") or params.get("assume_role"):
        params = get_assume_role(params)
    else:
        params = get_acs_connection_info(params)
    return params


def ecs_connect(module: AnsibleModule):
    """Return an ecs connection"""
    ecs_params = get_profile(module.params)
    # If we have a region specified, connect to its endpoint.
    region = module.params.get("alicloud_region")
    if region:
        try:
            ecs = connect_to_acs(footmark.ecs, region, **ecs_params)
        except AnsibleACSError as e:
            module.fail_json(msg=str(e))
    # Otherwise, no region so we fallback to the old connection method
    return ecs


def slb_connect(module: AnsibleModule):
    """Return an slb connection"""
    slb_params = get_profile(module.params)
    # If we have a region specified, connect to its endpoint.
    region = module.params.get("alicloud_region")
    if region:
        try:
            slb = connect_to_acs(footmark.slb, region, **slb_params)
        except AnsibleACSError as e:
            module.fail_json(msg=str(e))
    # Otherwise, no region so we fallback to the old connection method
    return slb


def dns_connect(module: AnsibleModule):
    """Return an dns connection"""
    dns_params = get_profile(module.params)
    # If we have a region specified, connect to its endpoint.
    region = module.params.get("alicloud_region")
    if region:
        try:
            dns = connect_to_acs(footmark.dns, region, **dns_params)
        except AnsibleACSError as e:
            module.fail_json(msg=str(e))
    # Otherwise, no region so we fallback to the old connection method
    return dns


def vpc_connect(module: AnsibleModule):
    """Return an vpc connection"""
    vpc_params = get_profile(module.params)
    # If we have a region specified, connect to its endpoint.
    region = module.params.get("alicloud_region")
    if region:
        try:
            vpc = connect_to_acs(footmark.vpc, region, **vpc_params)
        except AnsibleACSError as e:
            module.fail_json(msg=str(e))
    # Otherwise, no region so we fallback to the old connection method
    return vpc


def rds_connect(module: AnsibleModule):
    """Return an rds connection"""
    rds_params = get_profile(module.params)
    # If we have a region specified, connect to its endpoint.
    region = module.params.get("alicloud_region")
    if region:
        try:
            rds = connect_to_acs(footmark.rds, region, **rds_params)
        except AnsibleACSError as e:
            module.fail_json(msg=str(e))
    # Otherwise, no region so we fallback to the old connection method
    return rds


def ess_connect(module: AnsibleModule):
    """Return an ess connection"""
    ess_params = get_profile(module.params)
    # If we have a region specified, connect to its endpoint.
    region = module.params.get("alicloud_region")
    if region:
        try:
            ess = connect_to_acs(footmark.ess, region, **ess_params)
        except AnsibleACSError as e:
            module.fail_json(msg=str(e))
    # Otherwise, no region so we fallback to the old connection method
    return ess


def sts_connect(module: AnsibleModule):
    """Return an sts connection"""
    sts_params = get_profile(module.params)
    # If we have a region specified, connect to its endpoint.
    region = module.params.get("alicloud_region")
    if region:
        try:
            sts = connect_to_acs(footmark.sts, region, **sts_params)
        except AnsibleACSError as e:
            module.fail_json(msg=str(e))
    # Otherwise, no region so we fallback to the old connection method
    return sts


def ram_connect(module: AnsibleModule):
    """Return an ram connection"""
    ram_params = get_profile(module.params)
    # If we have a region specified, connect to its endpoint.
    region = module.params.get("alicloud_region")
    if region:
        try:
            ram = connect_to_acs(footmark.ram, region, **ram_params)
        except AnsibleACSError as e:
            module.fail_json(msg=str(e))
    # Otherwise, no region so we fallback to the old connection method
    return ram


def market_connect(module: AnsibleModule):
    """Return an market connection"""
    market_params = get_profile(module.params)
    # If we have a region specified, connect to its endpoint.
    region = module.params.get("alicloud_region")
    if region:
        try:
            market = connect_to_acs(footmark.market, region, **market_params)
        except AnsibleACSError as e:
            module.fail_json(msg=str(e))
    # Otherwise, no region so we fallback to the old connection method
    return market
